/*!
    @file  CDataGen.h
    @brief データ生成

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CDataGen_h
#define CDataGen_h

#include <complex>
#include <deque>
#include <fstream>
#include <boost/shared_ptr.hpp>
#include <cfloat>
#include "common.h"
#include "common_wrap.h"
#include "Util_wrap.h"

namespace FUJITSU {
class CDataGen : public AlgoBase {
public:
    /* コンストラクタ */
    CDataGen(const boost::shared_ptr<C_param_wrap> param);
    
    /* デストラクタ */
    virtual ~CDataGen();
    
    /* データ生成 */
    virtual void Execute(std::vector<std::complex<double> >& OutputData);
    
private:
    struct SDataGenParam {
        int                 WaveNum;      // 合成する波の数
        std::vector<double> WaveFreq;     // 合成する波の周波数[kHz]
        std::vector<double> WaveOffset;   // 合成する波のオフセット[ns]
        std::vector<double> WaveAmp;      // 合成する波の振幅比
        double              SamplingFreq; // サンプリング周波数
        int                 FftPoint;     // FFT点数(2のべき乗)
    };
    
    /*--------------
      メンバ変数
      --------------*/
    SDataGenParam                           param_;             /*!< パラメータ */
    
    /*--------------
      メンバ関数
      --------------*/
    
};
}

#endif /* CDataGen_h */
