/*!
    @file  CFft16r4.h
    @brief FFTサイズ16 radix-4 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CFft16r4_h
#define CFft16r4_h

#include <complex>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "CFftIF.h"

class CFft16r4 : public CFftIF {
public:
    /* コンストラクタ */
    CFft16r4(const unsigned int nfft,
              const bool ifft);
    
    /* デストラクタ */
    virtual ~CFft16r4();
    
    /*!   FFT Calculation         */
    virtual void fft(const std::vector<std::complex<double> >& In);

    /*! result get                */
    virtual const std::vector<std::complex<double> >& get() const;

    
private:
    unsigned int nFft_;                        /* FFT point Num */
    bool ifft_;                                /* true:IFFT, false:FFT */
    std::vector<std::complex<double> > xWork_; /* FFT work      */
    std::vector<std::complex<double> > W_;     /* 回転因子      */
    
    void RADIX_4(std::vector<std::complex<double> >& x, 
                 const unsigned int stage,
                 const unsigned int number);
};

#endif /* CFft16r4_h */
