/*!
    @file  CFft4096r64.h
    @brief FFTサイズ4096 radix-64 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CFft4096r64_h
#define CFft4096r64_h

#include <complex>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "CFftIF.h"

class CFft4096r64 : public CFftIF {
public:
    /* コンストラクタ */
    CFft4096r64(const unsigned int nfft,
              const bool ifft);
    
    /* デストラクタ */
    virtual ~CFft4096r64();
    
    /*!   FFT Calculation         */
    virtual void fft(const std::vector<std::complex<double> >& In);

    /*! result get                */
    virtual const std::vector<std::complex<double> >& get() const;

    
private:
    unsigned int nFft_;                        /* FFT point Num */
    bool ifft_;                                /* true:IFFT, false:FFT */
    std::vector<std::complex<double> > xWork_; /* FFT work      */
    std::vector<std::complex<double> > W_;     /* 回転因子      */
    
    void RADIX_64(std::vector<std::complex<double> >& x, 
                  const unsigned int stage,
                  const unsigned int number);
};

#endif /* CFft4096r64_h */
