/*!
    @file  CFft4r4.h
    @brief FFTサイズ4 radix-4 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CFft4r4_h
#define CFft4r4_h

#include <complex>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "CFftIF.h"

class CFft4r4 : public CFftIF {
public:
    /* コンストラクタ */
    CFft4r4(const unsigned int nfft,
              const bool ifft);
    
    /* デストラクタ */
    virtual ~CFft4r4();
    
    /*!   FFT Calculation         */
    virtual void fft(const std::vector<std::complex<double> >& In);

    /*! result get                */
    virtual const std::vector<std::complex<double> >& get() const;

    
private:
    unsigned int nFft_;                        /* FFT point Num */
    bool ifft_;                                /* true:IFFT, false:FFT */
    std::vector<std::complex<double> > xWork_; /* FFT work      */
    std::vector<std::complex<double> > W_;     /* 回転因子      */
};

#endif /* CFft4r4_h */
