/*!
    @file  CFft512r8.h
    @brief FFTサイズ512 radix-8 固定(ハード検討確認用)

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CFft512r8_h
#define CFft512r8_h

#include <complex>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "CFftIF.h"

class CFft512r8 : public CFftIF {
public:
    /* コンストラクタ */
    CFft512r8(const unsigned int nfft,
              const bool ifft);
    
    /* デストラクタ */
    virtual ~CFft512r8();
    
    /*!   FFT Calculation         */
    virtual void fft(const std::vector<std::complex<double> >& In);

    /*! result get                */
    virtual const std::vector<std::complex<double> >& get() const;

    
private:
    unsigned int nFft_;                        /* FFT point Num */
    bool ifft_;                                /* true:IFFT, false:FFT */
    std::vector<std::complex<double> > xWork_; /* FFT work      */
    std::vector<std::complex<double> > W_;     /* 回転因子      */
    
    void RADIX_8(std::vector<std::complex<double> >& x, 
                 const unsigned int stage,
                 const unsigned int number);
};

#endif /* CFft512r8_h */
