/*!
    @file  CFftIF.h
    @brief FFTインターフェース

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CFftIF_h
#define CFftIF_h

#include <complex>
#include <vector>
#include <boost/shared_ptr.hpp>

class CFftIF {
public:
    virtual ~CFftIF() {;}

    /*!   FFT Calculation         */
    virtual void fft(const std::vector<std::complex<double> >& In) {;};

    /*! result get                */
    virtual const std::vector<std::complex<double> >& get() const {;};

    /*!Report出力 */
    virtual void ReportStatisticalData(std::ostream& os, const double NowTime) {;};

protected:
void ComplexForLog(std::vector<std::complex<double> >& x)
{
    // 浮動小数点誤差が見にくいので消す(なくて良い処理)
    for (size_t i=0; i < x.size(); i++) {
        double re, im;
        if (x[i].real() > -0.000000001 && x[i].real() < 0.000000001) re = 0;
        else                                                         re = x[i].real();
        if (x[i].imag() > -0.000000001 && x[i].imag() < 0.000000001) im = 0;
        else                                                         im = x[i].imag();
        x[i] = std::complex<double>(re, im);
    }
}

//private:    
};

#endif /* CFftIF_h */
