/*!
    @file  CFftRef.h
    @brief Fast Fourier transform

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CFftRef_h
#define CFftRef_h

#include <complex>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "CFftIF.h"

class CFftRef : public CFftIF {
public:
    /* コンストラクタ */
    CFftRef(const unsigned int nfft, 
            const unsigned int radix,
            const bool ifft);
    
    /* デストラクタ */
    virtual ~CFftRef();
    
    /*!   FFT Calculation         */
    virtual void fft(const std::vector<std::complex<double> >& In);

    /*! result get                */
    virtual const std::vector<std::complex<double> >& get() const;

    
private:
    void fft_exec(std::vector<std::complex<double> >& x, const int length, const int radix, const bool ifft);

    //const unsigned int nFft_;                      /* FFT point Num */
    unsigned int nFft_;                      /* FFT point Num */
    unsigned int Radix_;                           /* 基数          */
    bool ifft_;                                    /* true:IFFT, false:FFT */
    std::vector<std::complex<double> > xWork_;     /* FFT work      */
};

#endif /* CFftRef_h */
