/*!
    @file  CFftWrap.h
    @brief Fast Fourier transform

    Copyright (C) 2017 Fujitsu Limited. All rights reserved.
*/

#ifndef CFftWrap_h
#define CFftWrap_h

#include <complex>
#include <deque>
#include <fstream>
#include <boost/shared_ptr.hpp>
#include <cfloat>
#include "common.h"
#include "common_wrap.h"
#include "Util_wrap.h"
#include "CFftIF.h"

namespace FUJITSU {
/* FFT処理 */
class CFftWrap : public AlgoBase {
public:
    /* コンストラクタ */
    CFftWrap(const boost::shared_ptr<C_param_wrap> param);
    
    /* デストラクタ */
    virtual ~CFftWrap();
    
    /* FFT処理実行 */
    virtual void Execute(const std::vector<std::complex<double> >& InputData,
                         std::vector<std::complex<double> >& OutputData,
                         const bool ifft);

    /* Report出力 */
    virtual void ReportStatisticalData(std::ostream& os, const double NowTime);
    
private:
    struct SFftParam {
        double              SamplingFreq; // サンプリング周波数(log出力に使用)
        int                 FftPoint;     // FFT点数(2のべき乗)
        int                 FftRadix;     // 基数
    };
    
    /*--------------
      メンバ変数
      --------------*/
    boost::shared_ptr<CFftIF>           fft_operator_;      /*!< FFTオブジェクト */
    boost::shared_ptr<CFftIF>           ifft_operator_;     /*!< FFTオブジェクト */
    SFftParam                           param_;             /*!< FFTパラメータ */
    boost::shared_ptr<std::ofstream>    fp_fft_in_;         /*!< FFT入力dump File Pointer */
    boost::shared_ptr<std::ofstream>    fp_fft_out_;        /*!< FFT出力dump File Pointer */
    boost::shared_ptr<std::ofstream>    fp_ifft_in_;        /*!< IFFT入力dump File Pointer */
    boost::shared_ptr<std::ofstream>    fp_ifft_out_;       /*!< IFFT出力dump File Pointer */
    
    /*--------------
      メンバ関数
      --------------*/
    /* log用に絶対値10-E7 以下を0に丸める */
    void ComplexForLog(std::vector<std::complex<double> >& x);
    /* paramチェック用関数 */
    bool   isPowerOfTwo(int a);
};
}

#endif /* CFftWrap_h */
