/*!
    @file  CRxFde.h
    @brief DSP Frequency Domain Equlizer Top(ヘッダ)

    @date     $Date:: 2022-06-24 11:46:52 #$
    @revision $Revision: 207 $

    Copyright (C) 2020 Fujitsu Limited. All rights reserved.
*/

/* 係数パラメータのDynamic設定は後回し */

#ifndef CRxFde_h
#define CRxFde_h

#include <complex>
#include <boost/shared_ptr.hpp>
#include "common.h"
#include "RXFDE_wrap.h"
#include "CRxFdeCore.h"

namespace FUJITSU {
/*! RxDSP 周波数ドメインイコライザ処理 */
class CRxFde : public IRXFDE {
public:
    /* コンストラクタ */
    CRxFde(const boost::shared_ptr<C_param_wrap> param);
    
    /* デストラクタ */
    virtual ~CRxFde();
    
    /* RxDSP 周波数ドメインイコライザ処理実行 */
    virtual void Execute(const double dsptm,
                         const double adjust_phase,
                         const std::complex<double>& InputData_H,
                         const std::complex<double>& InputData_V,
                         bool&                               OutputData_valid,
                         std::vector<std::complex<double> >& OutputData_H,
                         std::vector<std::complex<double> >& OutputData_V);

    /* 残留分散値設定処理 */
    virtual void SetResidualDispersion(const double rcd, const unsigned int scn);

    /* 残留周波数オフセット値設定処理 */
    virtual void SetResidualFrequencyOffset(const double rofs, const unsigned int scn);

    /* シミュレーション結果report */
    virtual void ReportStatisticalData(std::ostream& os, const double NowTime);

private:

    /*--------------
      メンバ変数
      --------------*/
    boost::shared_ptr<CRxFdeCore>    RxFdeCoreX_;
    boost::shared_ptr<CRxFdeCore>    RxFdeCoreY_;

    double rep_foff_;                    /* 周波数オフセット補償値(レポート用) */
    int    rep_foff_fft_grid_;           /* 周波数オフセット補償値 FFTグリッド単位(レポート用) */
    double rep_sample_time_;             /* 1sample時間[ps] (レポート用) */
    std::vector<double> rep_skew_;       /* SKEW補償値[ps] XI, XQ, YI, YQ */
    std::vector<double> rep_skew_samp_;  /* SKEW補償値[sample] XI, XQ, YI, YQ */
    
    /*--------------
      メンバ関数
      --------------*/
};

} /* namespace FUJITSU */

#endif /* CRxFde_h */
