/*!
    @file  CRxFdeFFT.h 
    @brief Radix2 FFT 

    @date     $Date:: 2022-01-21 10:47:24 #$
    @revision $Revision: 2 $

    Copyright (C) 2017-2020 Fujitsu Limited. All rights reserved.
*/

#ifndef CRXFDEFFT_H
#define CRXFDEFFT_H

#include <vector>
#include <complex>

namespace FUJITSU {
/*------------------------------------------------------*/
/*!   Radix=2 FFT Class define                           */
/*------------------------------------------------------*/
class CRxFdeFFT {
public:
    /*!   コンストラクタ          */
    CRxFdeFFT(const unsigned int nfft, const int dir);
    
    /*!   デストラクタ          */
    ~CRxFdeFFT();

    /*!   FFT Calculation         */
    void fft(const std::vector<std::complex<double> >& In);

    /*! result get                */
    const std::vector<std::complex<double> >& get() const;

private:
    const unsigned int nFft_;                      /* FFT point Num */
    unsigned int  iBit_;                           /* FFT Stage Num */
    std::vector<unsigned int> bsw_;                /* re-ordering table */
    std::vector<std::complex<double> > xCoef_;     /* Coef table */
    std::vector<std::complex<double> > xWork_;     /* FFT work */
};
}
#endif  /* CRXFDEFFT_H */
