/*!
    @file      RXFDE_wrap.cpp
    @brief     RXFDE wrapper 

    @date     $Date:: 2022-06-24 11:46:52 #$
    @revision $Revision: 207 $

    Copyright (C) 2020 Fujitsu Limited. All rights reserved.
*/

#include "RXFDE_wrap.h"
#include "createRXFDE_FUJITSU.h"

RXFDE_wrap::RXFDE_wrap(boost::shared_ptr<C_param_wrap> param) 
{
    std::string str, msg;
    str = "DSP_RXFDE_Block";
    if(1) {
        m_blk = createRXFDE_FUJITSU(param) ;
    } else {
        msg = "Parameter " + str + " is invalid.";
        g_err->appendMsg(msg, ERROR_MSG);
    }
}

void RXFDE_wrap::ReportStatisticalData(std::ostream& os, const double NowTime) {
    m_blk->ReportStatisticalData(os, NowTime) ;
}
const std::string RXFDE_wrap::getVender() {
    return std::string("FUJITSU") ;
}
const std::string RXFDE_wrap::getMakeTime() {
    return std::string(__DATE__) + std::string(" ") + std::string(__TIME__) ;
}

void RXFDE_wrap::Execute(const double dsptm,
                         const double adjust_phase,
                         const std::complex<double>& InputData_H,
                         const std::complex<double>& InputData_V,
                         bool&                               OutputData_valid,
                         std::vector<std::complex<double> >& OutputData_H,
                         std::vector<std::complex<double> >& OutputData_V) {
    m_blk->Execute(dsptm, adjust_phase, InputData_H, InputData_V, OutputData_valid, OutputData_H, OutputData_V);
}

void RXFDE_wrap::SetResidualDispersion(const double rcd, const unsigned int scn) {
    m_blk->SetResidualDispersion(rcd, scn);
}

void RXFDE_wrap::SetResidualFrequencyOffset(const double rofs, const unsigned int scn) {
    m_blk->SetResidualFrequencyOffset(rofs, scn);
}

