/*!
    @file      RXFDE_wrap.h
    @brief     RXFDE wrapper Header File

    @date     $Date:: 2022-06-24 11:46:52 #$
    @revision $Revision: 207 $

    Copyright (C) 2020 Fujitsu Limited. All rights reserved.
*/

#ifndef __RXFDE_WRAP_H__
#define __RXFDE_WRAP_H__
/* 公開Header */
#include <complex>
#include <vector>
#include <boost/shared_ptr.hpp>
#include "Util_wrap.h"

class IRXFDE {
public:
    virtual ~IRXFDE() {};
    virtual void ReportStatisticalData(std::ostream& os, const double NowTime) = 0;
    virtual void Execute(
                 const double dsptm,
                 const double adjust_phase,
                 const std::complex<double>& InputData_H,
                 const std::complex<double>& InputData_V,
                 bool&                               OutputData_valid,
                 std::vector<std::complex<double> >& OutputData_H,
                 std::vector<std::complex<double> >& OutputData_V) = 0;
    virtual void SetResidualDispersion(const double rcd, const unsigned int scn) = 0;
    virtual void SetResidualFrequencyOffset(const double rofs, const unsigned int scn)= 0;
};

class RXFDE_wrap {
public:
    RXFDE_wrap() {;}
    RXFDE_wrap(boost::shared_ptr<C_param_wrap> param) ;
    void ReportStatisticalData(std::ostream& os, const double NowTime) ;
    const std::string getVender() ;
    const std::string getMakeTime() ;
    void Execute(const double dsptm,
                 const double adjust_phase,
                 const std::complex<double>& InputData_H,
                 const std::complex<double>& InputData_V,
                 bool&                               OutputData_valid,
                 std::vector<std::complex<double> >& OutputData_H,
                 std::vector<std::complex<double> >& OutputData_V);
    void SetResidualDispersion(const double rcd, const unsigned int scn);
    void SetResidualFrequencyOffset(const double rofs, const unsigned int scn);

private:
    boost::shared_ptr<IRXFDE> m_blk ;
};

#endif
