/*!
    @file  C_ErrMng.h
    @brief エラー管理クラス 
 
    @date     $Date:: 2016-01-29 16:40:19 #$
    @revision $Revision: 8 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef ERROR_MNG_H
#define ERROR_MNG_H

#include <string>
#include <iostream>
#include <sstream>
#ifndef rel_NTT
#include <vector>
#endif
#include <boost/shared_ptr.hpp>


enum ERROR_SEVERITY {
  INFO_MSG     = 0 , /*!< Informationメッセージです。checkStatus()でシステムは止めません。 */
  WARNING_MSG  = 4 , /*!< Warningメッセージです。checkStatus()でシステムは止めません。 */
  ERROR_MSG    = 8   /*!< Errorメッセージです。checkStatus()でシステムを止めます。 */
} ;


class ErrorMng_wrap {
public:
  ErrorMng_wrap() : m_os1(NULL), m_os2(NULL) { ; }
  virtual ~ErrorMng_wrap() { ; }
  
  virtual const std::string getVender() = 0 ;
  virtual const std::string getMakeTime() = 0 ;
  
  /*! 
   *  @brief メッセージとメッセージのセヴィリティを登録します。 
   *  @param msg   メッセージ本文です。 
   *  @param sev   メッセージのセヴィリティです。 
   */
  virtual void appendMsg(const std::string& msg, ERROR_SEVERITY sev = ERROR_MSG) = 0 ;

  /*! 
   *  @brief メッセージのセヴィリティからシステムを継続するか判定します。 
   */
  virtual void checkStatus() = 0 ;

  /*!
   * @brief メッセージ出力先を設定します。 
            注意）openしていないファイルは、出力されません。openしてsetして下さい。 
     例1) setOstreamOne(&std::cout)
     例2) std::ofstream* fp_repo = new std::ofstream;
          fp_repo->open(＊＊＊) ;
          setOstreamTwo(fp_repo)
   */
  virtual void setOstreamOne(std::ostream* os) = 0 ;
  virtual void setOstreamTwo(std::ostream* os) = 0 ;

#ifndef rel_NTT
  /* =================================================
     以降の関数は、シナリオ解釈クラスSc_Ctrl専用 
     ================================================= */
  /*! (シナリオ解釈クラスSc_Ctrl専用) メッセージ出力/セビリティの設定関数 
   */
  virtual void appendMsgSc(      int                       msg_no, 
                           const std::string&              file_name, 
                                 int                       line_no, 
                           const std::vector<std::string>& add_str_vec = std::vector<std::string>() ) = 0 ;
#endif  
  
protected:
  std::ostream* m_os1 ;
  std::ostream* m_os2 ;
} ;


/*! @brief ErrorMng_wrapクラスをインスタンスします 
 *  @return ErrorMng_wrapクラスのshared_ptr 
 */
boost::shared_ptr<ErrorMng_wrap> createErrorMng() ;

extern boost::shared_ptr<ErrorMng_wrap> g_err ;

#endif
