/*!
    @file  OneBlockBase.h
    @brief DSP関係の共通クラス定義 

    @date     $Date:: 2016-06-10 12:47:02 #$
    @revision $Revision: 242 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef ONE_BLOCK_BASE_H
#define ONE_BLOCK_BASE_H

#include <map>
#include <string>
#include <complex>
#include <boost/shared_ptr.hpp>
#include "I_param.h"

class AlgoBase ;
class AlgoBaseTx ;

/* DSP用の共通クラス(OneBlockBase, AlgoBase) */
class OneBlockBase {
public:
  OneBlockBase() { ; }
  virtual ~OneBlockBase() { ; }
  virtual void ReportStatisticalData(std::ostream& os, const double NowTime) = 0;
  virtual const std::string getVender() = 0;
  virtual const std::string getMakeTime() = 0;
// MCC_FUJITSU Execute I/F 
// TS_SYNC_FUJITSU Execute I/F 
  virtual void Execute(const double                dsptm,
                       const bool                  FrameTopIn,
                       const std::complex<double>& InputData_X,
                       const std::complex<double>& InputData_Y) ;
// FMR_FUJITSU  Execute I/F 
  virtual void Execute(const double                dsptm,
                       const std::complex<double>& InputData_H,
                       const std::complex<double>& InputData_V,
                             std::complex<double>& OutputData_H,
                             std::complex<double>& OutputData_V );
// CPR_FUJITSU  Execute I/F 
// NPCC_FUJITSU Execute I/F 
  virtual void Execute(const double                dsptm,
                       const bool                  FrameTopIn,
                       const std::complex<double>& InputData_H,
                       const std::complex<double>& InputData_V,
                             bool&                 FrameTopOut,
                             std::complex<double>& OutputData_H,
                             std::complex<double>& OutputData_V);
// DEC_FUJITSU Execute I/F 
  virtual void Execute(const double                dsptm,
                       const bool                  FrameTopIn,
                       const std::complex<double>& InputData_H,
                       const std::complex<double>& InputData_V,
                             bool&                 FrameTopOut,
                             std::vector<unsigned int>& OutputBitData);
// PHA_FUJITSU Execute I/F 
  virtual void Execute(const double                dsptm,
                       const double&               AdjustPhase,
                       const std::complex<double>& InputData_H,
                       const std::complex<double>& InputData_V,
                             bool&                 IfftFirstData,
                             std::complex<double>& OutputData_H,
                             std::complex<double>& OutputData_V ) ;
// SS_FUJITSU Execute I/F 
  virtual void Execute(const double                dsptm,
                       const int                   VcoInitialTraining,
                       const unsigned int          PhaP_OffsetOn,
                       const bool                  PhaFirstData,
#ifndef rel_NTT
                       const unsigned int          rxrefck,
#endif
                       const std::complex<double>& InputData_H,
                       const std::complex<double>& InputData_V,
                             bool&                 AdjustPhaseOutTiming,
                             double&               AdjustPhase,
                             double&               ExtCtrl,
                             bool&                 VcoFeedbackTiming,
                             double&               VcoFeedback,
                             double&               ClkVcoDo) ;
// SQM_FUJITSU Execute I/F 
  virtual void Execute(const double                dsptm,
                       const bool                  FrameTopIn,
                       const std::complex<double>& InputData_H,
                       const std::complex<double>& InputData_V,
                             bool&                 Calc_Status ) ;
#ifndef rel_NTT
// MCC_FUJITSU Execute I/F 
  virtual void Execute(const double                dsptm,
                       const bool                  FrameTopIn,
                       const std::complex<double>& InputData_H,
                       const std::complex<double>& InputData_V,
                             int*                  monitor ) ;
#endif

    // FFT Execute I/F
    virtual void Execute(const std::vector<std::complex<double> >& InputData,
                         std::vector<std::complex<double> >& OutputData,
                         const bool ifft);
                         
    // DTAT_GEN Execute I/F
    virtual void Execute(std::vector<std::complex<double> >& OutputData);
    

  virtual void Get(double& Pow_Ave_H,
                   double& Dif_Ave_H,
                   double& Pow_Ave_V,
                   double& Dif_Ave_V) ;
  virtual void SetAeqTsTiming(const double ats_timing_h, const double ats_timing_v);
  virtual void SetFrequencyOffset(const double fo_h_in, const double fo_v_in);
  virtual void read_PD_MON_FOR_FEQ(double& val, bool& flg);
protected:
  /*  機能ブロックのアルゴリズム(ポインタ) */
  boost::shared_ptr<AlgoBase> m_algo ;
} ;

class AlgoBase {
public:
  enum SignalPolarization {
      H,
      V
  };

  AlgoBase() { ; }
  AlgoBase(boost::shared_ptr<I_param> /*!< param */) { ; }
  virtual ~AlgoBase() { ; }
  virtual void ReportStatisticalData(std::ostream& /*!< os */, const double /*!< NowTime */) { ; }
// MCC_FUJITSU Execute I/F 
// TS_SYNC_FUJITSU Execute I/F 
  virtual void Execute(const double                /*!< dsptm */,
                       const bool                  /*!< FrameTopIn */,
                       const std::complex<double>& /*!< InputData_X */,
                       const std::complex<double>& /*!< InputData_Y */) { ; }
// FMR_FUJITSU  Execute I/F 
  virtual void Execute(const double                /*!< dsptm */,
                       const std::complex<double>& /*!< InputData_H */,
                       const std::complex<double>& /*!< InputData_V */,
                             std::complex<double>& /*!< OutputData_H */,
                             std::complex<double>& /*!< OutputData_V */) { ; }
// CPR_FUJITSU  Execute I/F 
// NPCC_FUJITSU Execute I/F 
  virtual void Execute(const double                /*!< dsptm */,
                       const bool                  /*!< FrameTopIn */,
                       const std::complex<double>& /*!< InputData_H */,
                       const std::complex<double>& /*!< InputData_V */,
                             bool&                 /*!< FrameTopOut */,
                             std::complex<double>& /*!< OutputData_H */,
                             std::complex<double>& /*!< OutputData_V */) { ; }
// DEC_FUJITSU Execute I/F 
  virtual void Execute(const double                /*!< dsptm */,
                       const bool                  /*!< FrameTopIn */,
                       const std::complex<double>& /*!< InputData_H */,
                       const std::complex<double>& /*!< InputData_V */,
                             bool&                 /*!< FrameTopOut */,
                             std::vector<unsigned int>&    /*!< OutputBitData */) { ; }
// PHA_FUJITSU Execute I/F 
  virtual void Execute(const double                /*!< dsptm */,
                       const double&               /*!< AdjustPhase */,
                       const std::complex<double>& /*!< InputData_H */,
                       const std::complex<double>& /*!< InputData_V */,
                             bool&                 /*!< IfftFirstData */,
                             std::complex<double>& /*!< OutputData_H */,
                             std::complex<double>& /*!< OutputData_V */) { ; }
// SS_FUJITSU Execute I/F 
  virtual void Execute(const double                /*!< dsptm */,
                       const int                   /*!< VcoInitialTraining */,
                       const unsigned int          /*!< PhaP_OffsetOn */,
                       const bool                  /*!< PhaFirstData */,
#ifndef rel_NTT
                       const unsigned int          /*!< rxrefck */,
#endif
                       const std::complex<double>& /*!< InputData_H */,
                       const std::complex<double>& /*!< InputData_V */,
                             bool&                 /*!< AdjustPhaseOutTiming */,
                             double&               /*!< AdjustPhase */,
                             double&               /*!< ExtCtrl */,
                             bool&                 /*!< VcoFeedbackTiming */,
                             double&               /*!< VcoFeedback */, 
                             double&               /*!< ClkVcoDo */) { ; }
// SQM_FUJITSU Execute I/F 
  virtual void Execute(const double                /*!< dsptm */,
                       const bool                  /*!< FrameTopIn */,
                       const std::complex<double>& /*!< InputData_H */,
                       const std::complex<double>& /*!< InputData_V */,
                             bool&                 /*!< Calc_Status */ ) { ; }
#ifndef rel_NTT
// MCC_FUJITSU Execute I/F 
  virtual void Execute(const double                /*!< dsptm */,
                       const bool                  /*!< FrameTopIn */,
                       const std::complex<double>& /*!< InputData_H */,
                       const std::complex<double>& /*!< InputData_V */,
                             int*                  /*!< monitor */ ) { ; }
#endif

    // FFT Execute I/F
    virtual void Execute(const std::vector<std::complex<double> >& InputData,
                         std::vector<std::complex<double> >& OutputData,
                         const bool ifft) { ; }

    // DTAT_GEN Execute I/F
    virtual void Execute(std::vector<std::complex<double> >& OutputData) { ; }

  virtual void Get(double& /*!< Pow_Ave_H */,
                   double& /*!< Dif_Ave_H */,
                   double& /*!< Pow_Ave_V */,
                   double& /*!< Dif_Ave_V */) { ; }
  virtual void SetAeqTsTiming(const double /*!< ats_timing */) { ; }
  virtual void SetAeqTsTiming(const double /*!< ats_timing_h */, const double /*!< ats_timing_v */) { ; }
  virtual void SetFrequencyOffset(const double /*!< fo_h_in */, const double /*!< fo_v_in */) { ; }
  virtual void read_PD_MON_FOR_FEQ(double& /*!< val */ , bool& /*!< flg */ ) { ; }
protected:
  /*  パラメータ(ポインタ) */
  boost::shared_ptr<I_param> m_param ;
} ;

/* TxDSP用の共通クラス(OneBlockBaseTx, AlgoBaseTx) */
class OneBlockBaseTx {
public:
    enum eHybSmblKind { K_UNKNOWN = 0, K_A, K_B, K_C, K_P, K_T }; /* ハイブリッド変調モード時のシンボル種類 */
    struct STxDspInput {
        bool valid;                    /* 2倍オーバーサンプリング入力のValid信号 */
        std::vector<unsigned int> x;   /* TxDSP入力(変調方式に応じたx側の1symbol分bit列) */
        std::vector<unsigned int> y;   /* TxDSP入力(変調方式に応じたy側の1symbol分bit列) */
        eHybSmblKind Kind_x;            /* ハイブリッド変調モード時のシンボル種類 */
        eHybSmblKind Kind_y;            /* ハイブリッド変調モード時のシンボル種類 */
    };

    OneBlockBaseTx() { ; }
    virtual ~OneBlockBaseTx() { ; }
    virtual void ReportStatisticalData(std::ostream& os, const double NowTime) = 0;
    virtual const std::string getVender() = 0;
    virtual const std::string getMakeTime() = 0;

// MAP_FUJITSU Execute I/F 
    virtual void Execute(const double                dsptm,
                         const bool                  ClockTop,
                         const struct OneBlockBaseTx::STxDspInput& Input,
                               std::complex<double>& OutputData_X,
                               std::complex<double>& OutputData_Y);
// TSS_FUJITSU Execute I/F 
// FC_FUJITSU Execute I/F 
// EQL_FUJITSU Execute I/F 
    virtual void Execute(const double                dsptm,
                         const bool                  ClockTop,
                         const std::complex<double>& InputData_X,
                         const std::complex<double>& InputData_Y,
                               std::complex<double>& OutputData_X,
                               std::complex<double>& OutputData_Y );
// PreCoder_FUJITSU Execute I/F 
    virtual void Execute(const double            dsptm,
                         const struct OneBlockBaseTx::STxDspInput&   Input,
                               struct OneBlockBaseTx::STxDspInput&   Output,
                               bool&             ClockTop);
protected:
    /*  機能ブロックのアルゴリズム(ポインタ) */
    boost::shared_ptr<AlgoBaseTx> m_algo ;
} ;

class AlgoBaseTx {
public:
    enum SignalPolarization {
        X,
        Y
    };

    AlgoBaseTx() { ; }
    AlgoBaseTx(boost::shared_ptr<I_param> param) { ; }
    virtual ~AlgoBaseTx() { ; }
    virtual void ReportStatisticalData(std::ostream& /*os*/, const double /*NowTime*/) { ; }

// MAP_FUJITSU Execute I/F 
    virtual void Execute(const double                /*dsptm*/,
                         const bool                  /*ClockTop*/,
                         const struct OneBlockBaseTx::STxDspInput&   /*Input*/,
                               std::complex<double>& /*OutputData_X*/,
                               std::complex<double>& /*OutputData_Y*/) { ; }
// TSS_FUJITSU Execute I/F 
// FC_FUJITSU Execute I/F  
// EQL_FUJITSU Execute I/F 
    virtual void Execute(const double                /*dsptm*/,
                         const bool                  /*ClockTop*/,
                         const std::complex<double>& /*InputData_X*/,
                         const std::complex<double>& /*InputData_Y*/,
                               std::complex<double>& /*OutputData_X*/,
                               std::complex<double>& /*OutputData_Y*/) { ; }
// PreCoder_FUJITSU Execute I/F 
    virtual void Execute(const double            /*dsptm*/,
                         const struct OneBlockBaseTx::STxDspInput&   /*Input*/,
                               struct OneBlockBaseTx::STxDspInput&   /*Output*/,
                               bool&             /*ClockTop*/) { ; }
protected:
    /*  パラメータ(ポインタ) */
    boost::shared_ptr<I_param> m_param ;
} ;

#endif
