/*! 
    @file  C_err_msg.h 
    @brief エラーメッセージの定義 
 
    @date     $Date:: 2016-01-28 13:10:50 #$ 
    @revision $Revision: 2 $ 
 
    Copyright (C) 2015 Fujitsu Limited. All rights reserved. 
*/

#ifndef C_ERR_MSG_H
#define C_ERR_MSG_H

#include <map>
#include <string>

/*! 
  エラーメッセージ管理クラス 
  @brief  エラーメッセージを管理する機能を持つ 
 */
class C_err_msg {
public:
    /* クラス内定義 */

    /* エラーの種類とエラーコードを定義 */
    enum E_err_type {
        /* C_token_list 内エラー */
        TOKEN_LIST_ERR_ARRAY_IDX = 10,
        TOKEN_LIST_ERR_PARAM_NAME,
        TOKEN_LIST_ERR_VAL_STR,
        TOKEN_LIST_ERR_ARRAY_RANGE,

        /* C_parser_base 内エラー */
        PARSER_BASE_ERR_OPEN_FILE = 20,
        PARSER_BASE_ERR_EMPTY_FILE,
        PARSER_BASE_ERR_ZENAKKU,
        PARSER_BASE_ERR_NO_EXIST_HEADER,

        /* C_param_parser 内エラー */
        PARAM_PARSER_ERR_NO_EXIST_CODE_END = 30,
        PARAM_PARSER_ERR_NO_EXIST_PARAM_DEF_END,
        PARAM_PARSER_ERR_EXIST_OTHER_PARAM_DEF_END,

        /* C_sc_parser 内エラー */
        SC_PARSER_ERR_NO_EXIST_CODE_END = 40,
        SC_PARSER_ERR_UNDEF_CMD,
        SC_PARSER_ERR_CMD_FORMAT,
        SC_PARSER_ERR_INSIDE_DEF,
        SC_PARSER_ERR_OUTSIDE_DEF_END,
        SC_PARSER_ERR_ASSIGN_TYPE_ERROR,
        SC_PARSER_ERR_COMP_TYPE_ERROR,
        SC_PARSER_ERR_SC_REDEF,       
        SC_PARSER_ERR_SC_UNDEF,
        SC_PARSER_ERR_SUB_REDEF,       
        SC_PARSER_ERR_SUB_UNDEF,
        SC_PARSER_ERR_LABEL_REDEF,
        SC_PARSER_ERR_LABEL_UNDEF,
        SC_PARSER_ERR_ILLEGAL_JUMP,
        SC_PARSER_ERR_NOT_FOUND_LOOP_END,
        SC_PARSER_ERR_OUTSIDE_DEF_CMD,

        /* C_dset_cmd_interpreter 内エラー */ 
        DSET_CMD_INTERPRETER_ERR_NO_EXIST_EQUAL = 100,
        DSET_CMD_INTERPRETER_ERR_PARAM_NAME,
        DSET_CMD_INTERPRETER_ERR_VAL,
        DSET_CMD_INTERPRETER_ERR_SIZE_OF_VECTOR,
        DSET_CMD_INTERPRETER_ERR_TYPE_OF_PARAM,
        DSET_CMD_INTERPRETER_ERR_TYPE_OF_ARRAY,
        DSET_CMD_INTERPRETER_ERR_STATE_OF_STATIC_PARAM,
        DSET_CMD_INTERPRETER_ERR_STATE_OF_MONITOR_PARAM,
        DSET_CMD_INTERPRETER_ERR_STATE_OF_NONSTATE_PARAM,
        DSET_CMD_INTERPRETER_ERR_ARGS_NUM,

        /* C_rset_cmd_interpreter 内エラー */
        RSET_CMD_INTERPRETER_ERR_NO_EXIST_PLUS_EQUAL = 110,
        RSET_CMD_INTERPRETER_ERR_PARAM_NAME,
        RSET_CMD_INTERPRETER_ERR_VAL,
        RSET_CMD_INTERPRETER_ERR_SIZE_OF_VECTOR,
        RSET_CMD_INTERPRETER_ERR_TYPE_OF_PARAM,
        RSET_CMD_INTERPRETER_ERR_TYPE_OF_ARRAY,
        RSET_CMD_INTERPRETER_ERR_STATE_OF_STATIC_PARAM,
        RSET_CMD_INTERPRETER_ERR_STATE_OF_MONITOR_PARAM,
        RSET_CMD_INTERPRETER_ERR_STATE_OF_NONSTATE_PARAM,
        RSET_CMD_INTERPRETER_ERR_ARGS_NUM,

        /* C_tdset_cmd_interpreter 内エラー */
        TDSET_CMD_INTERPRETER_ERR_ARGS_NUM = 120,
        TDSET_CMD_INTERPRETER_ERR_TIME_VAR_POS,
        TDSET_CMD_INTERPRETER_ERR_NO_INTEGER_ARG,

        /* C_trset_cmd_interpreter 内エラー */
        TRSET_CMD_INTERPRETER_ERR_ARGS_NUM = 130,
        TRSET_CMD_INTERPRETER_ERR_TIME_VAR_POS,
        TRSET_CMD_INTERPRETER_ERR_WAIT_CMD_POS,
        TRSET_CMD_INTERPRETER_ERR_NO_INTEGER_ARG,
        TRSET_CMD_INTERPRETER_ERR_PLUS_POS,

        /* C_for_cmd_interpreter 内エラー */
        FOR_CMD_INTERPRETER_ERR_ARGS_NUM = 140,
        FOR_CMD_INTERPRETER_ERR_FOR_CMD_POS,
        FOR_CMD_INTERPRETER_ERR_NO_INTEGER_ARG,

        /* C_wait_cmd_interpreter 内エラー */
        WAIT_CMD_INTERPRETER_ERR_ARGS_NUM = 150,
        WAIT_CMD_INTERPRETER_ERR_WAIT_CMD_POS,
        WAIT_CMD_INTERPRETER_ERR_LEFT_PARENTHETIC,
        WAIT_CMD_INTERPRETER_ERR_RIGHT_PARENTHETIC,
        WAIT_CMD_INTERPRETER_ERR_COND_LEFT,
        WAIT_CMD_INTERPRETER_ERR_COND_RIGHT,
        WAIT_CMD_INTERPRETER_ERR_COND_OP,
        WAIT_CMD_INTERPRETER_ERR_NO_INTEGER_ARG,
        WAIT_CMD_INTERPRETER_ERR_TIMEOUT_TYPE,
        WAIT_CMD_INTERPRETER_ERR_COND_TYPE,

        /* C_jump_cmd_interpreter 内エラー */
        JUMP_CMD_INTERPRETER_ERR_ARGS_NUM = 160,
        JUMP_CMD_INTERPRETER_ERR_JUMP_CMD_POS,
        JUMP_CMD_INTERPRETER_ERR_LEFT_PARENTHETIC,
        JUMP_CMD_INTERPRETER_ERR_RIGHT_PARENTHETIC,
        JUMP_CMD_INTERPRETER_ERR_COND_LEFT,
        JUMP_CMD_INTERPRETER_ERR_COND_RIGHT,
        JUMP_CMD_INTERPRETER_ERR_COND_OP,
        JUMP_CMD_INTERPRETER_ERR_NO_INTEGER_ARG,
        JUMP_CMD_INTERPRETER_ERR_COND_TYPE,

        /* C_call_cmd_interpreter 内エラー */
        CALL_CMD_INTERPRETER_ERR_ARGS_NUM = 170,
        CALL_CMD_INTERPRETER_ERR_CALL_CMD_POS,

        /* C_scdef_cmd_interpreter 内エラー */
        SCDEF_CMD_INTERPRETER_ERR_ARGS_NUM = 180,
        SCDEF_CMD_INTERPRETER_ERR_SCDEF_CMD_POS,
        SCDEF_CMD_INTERPRETER_ERR_REDEF_SCENARIO,

        /* C_scend_cmd_interpreter 内エラー */
        SCEND_CMD_INTERPRETER_ERR_ARGS_NUM = 190,
        SCEND_CMD_INTERPRETER_ERR_SCEND_CMD_POS,

        /* C_subdef_cmd_interpreter 内エラー */
        SUBDEF_CMD_INTERPRETER_ERR_ARGS_NUM = 200,
        SUBDEF_CMD_INTERPRETER_ERR_SUBDEF_CMD_POS,
        SUBDEF_CMD_INTERPRETER_ERR_REDEF_SUB,

        /* C_subend_cmd_interpreter 内エラー */
        SUBEND_CMD_INTERPRETER_ERR_ARGS_NUM = 210,
        SUBEND_CMD_INTERPRETER_ERR_SUBEND_CMD_POS,

        /* C_labeldef_cmd_interpreter 内エラー */
        LABELDEF_CMD_INTERPRETER_ERR_ARGS_NUM = 220,
        LABELDEF_CMD_INTERPRETER_ERR_LABELDEF_CMD_POS,
        LABELDEF_CMD_INTERPRETER_ERR_REDEF_LABEL,

        /* C_simstop_cmd_interpreter 内エラー */
        SIMSTOP_CMD_INTERPRETER_ERR_ARGS_NUM = 230,
        SIMSTOP_CMD_INTERPRETER_ERR_SIMSTOP_CMD_POS,

        /* C_scenairo 内エラー */
        SCENARIO_ERR_SUBSTACK_EMPTY = 300,
        SCENARIO_ERR_SUBSTACK_OVERFLOW,

        /* C_scenairo_elem 内エラー */
        SCENARIO_ELEM_ERR_UNDEF_CMD = 310,
        SCENARIO_ELEM_ERR_CMD_EXECUTE,
        SCENARIO_ELEM_ERR_ASSIGN_TYPE,
        SCENARIO_ELEM_ERR_COMP_TYPE,
        SCENARIO_ELEM_ERR_WAIT_TIME_OVER,
        SCENARIO_ELEM_ERR_JUMP_COUNT_OVER,

        /* C_param 内エラー */
        PARAM_ERR_FIND_NAME = 400,
        PARAM_ERR_TRANS_INT,
        PARAM_ERR_TRANS_DOUBLE,
        PARAM_ERR_TRANS_STRING,
        PARAM_ERR_TRANS_COMPLEX,
        PARAM_ERR_TRANS_INT_VEC,
        PARAM_ERR_TRANS_DOUBLE_VEC,
        PARAM_ERR_TRANS_STRING_VEC,
        PARAM_ERR_TRANS_COMPLEX_VEC,
        PARAM_ERR_TYPE_DECIDED,
        PARAM_ERR_TRANS_UNSIGNED_INT,
        PARAM_ERR_TRANS_UNSIGNED_INT_VEC,

        /* C_Util内エラー */
        UTIL_ERR_PARAM_ACQUIRED  = 10000,
        UTIL_ERR_OUTPUTFILE_OPEN,
        UTIL_ERR_ABORT_PARAM
    };

    /* ワーニングの種類とワーニングコードを定義 */
    enum E_warn_type {
        /* C_scenairo 内ワーニング */
        SCENARIO_WARN_NEXT_TIME = 10,

        /* C_scenairo_elem 内ワーニング */
        SCENARIO_ELEM_WARN_WAIT_TIME_OVER = 20,

        /* C_sc_parser 内ワーニング */
        SC_PARSER_WARN_MULTI_DEF_SC = 40,

        /* C_param 内ワーニング */
        PARAM_WARN_FIND_NAME = 400,
        PARAM_WARN_TRANS_INT,
        PARAM_WARN_TRANS_DOUBLE,
        PARAM_WARN_TRANS_STRING,
        PARAM_WARN_TRANS_COMPLEX,
        PARAM_WARN_TRANS_INT_VEC,
        PARAM_WARN_TRANS_DOUBLE_VEC,
        PARAM_WARN_TRANS_STRING_VEC,
        PARAM_WARN_TRANS_COMPLEX_VEC,
        PARAM_WARN_TYPE_DECIDED,
        PARAM_WARN_TRANS_UNSIGNED_INT,
        PARAM_WARN_TRANS_UNSIGNED_INT_VEC,

        /* C_Util内ワーニング */
        UTIL_WARN_PARAM_ACQUIRED  = 10000,
        UTIL_WARN_OUTPUTFILE_OPEN,
        UTIL_WARN_ABORT_PARAM
    };

    /* インフォメーションの種類とインフォメーションコードを定義 */
    enum E_info_type {
        /* C_scenairo内インフォメーション */
        SCENARIO_INFO_STOP = 10,
        SCENARIO_INFO_END,

        /* C_param 内インフォメーション */
        PARAM_INFO_FIND_NAME = 400,
        PARAM_INFO_TRANS_INT,
        PARAM_INFO_TRANS_DOUBLE,
        PARAM_INFO_TRANS_STRING,
        PARAM_INFO_TRANS_COMPLEX,
        PARAM_INFO_TRANS_INT_VEC,
        PARAM_INFO_TRANS_DOUBLE_VEC,
        PARAM_INFO_TRANS_STRING_VEC,
        PARAM_INFO_TRANS_COMPLEX_VEC,
        PARAM_INFO_TYPE_DECIDED,
        PARAM_INFO_TRANS_UNSIGNED_INT,
        PARAM_INFO_TRANS_UNSIGNED_INT_VEC,

        /* C_Util内のインフォメーション */
        UTIL_INFO_PARAM_ACQUIRED  = 10000,
        UTIL_INFO_OUTPUTFILE_OPEN,
        UTIL_INFO_ABORT_PARAM
    };

private:
    /* メンバ変数 */

    /* エラーメッセージマップ */
    static std::map<E_err_type, std::string> m_err_msg_map;

    /* ワーニングメッセージマップ */
    static std::map<E_warn_type, std::string> m_warn_msg_map;

    /* インフォメーションメッセージマップ */
    static std::map<E_info_type, std::string> m_info_msg_map;

    /* 初期化フラグ */
    static bool m_init_flag;

public:

    /*! @brief   コンストラクタ 
     */
    C_err_msg();

    /*! @brief   エラーメッセージ取得関数 
     *  @param   err_code エラーコード (C_err_msg::E_err_type) 
     *  @return  エラーメッセージ 
     */
    std::string getErrMsg(E_err_type err_code);

    /*! @brief   ワーニングメッセージ取得関数 
     *  @param   warn_code ワーニングコード (C_err_msg::E_warn_type) 
     *  @return  ワーニングメッセージ 
     */
    std::string getWarnMsg(E_warn_type warn_code);

    /*! @brief   インフォメーションメッセージ取得関数 
     *  @param   info_code インフォメーションコード (C_err_msg::E_info_type) 
     *  @return  インフォメーションメッセージ 
     */
    std::string getInfoMsg(E_info_type info_code);

    /*! @brief   メッセージの%sをstrに変換 
     *  @param str     置換対象文字列 
     *  @param chgstr  置換文字列 
     *  @return 置換後文字列 
     */
    std::string changeString(std::string str, std::string chgstr);

private:

    /*! @brief   エラーメッセージマップ作成関数 
     */
    void createErrMsgMap();

    /*! @brief   ワーニングメッセージマップ作成関数 
     */
    void createWarnMsgMap();

    /*! @brief   インフォメーションメッセージマップ作成関数 
     */
    void createInfoMsgMap();
};

#endif
