/*! 
    @file  C_for_cmd_interpreter.h 
    @brief FORコマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_FOR_CMD_INTERPRETER_H
#define C_FOR_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"

/*! 
  FORコマンドインタプリタクラス 
  @brief  FORコマンドの解釈機能を持つ 
 */
class C_for_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    bool m_check;
public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_for_cmd_interpreter(boost::shared_ptr<I_param> param,
                          boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_for_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   ループ変数初期化コマンドを生成する 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createInitLoopVarCmd(int line_num);


    /*! @brief   ループ先頭のシナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createLoopBeginCmd(std::vector<std::string>::iterator begin,
                                       std::vector<std::string>::iterator end,
                                       int line_num);

    /*! @brief   ループ終端のシナリオ要素を生成する 
     *  @return  シナリオ要素 
     */
    C_scenario_elem createLoopEndCmd(int line_num);

    /*! @brief   FORコマンドラベル名の取得を行う 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  ラベル名 
     */
    std::string getLabelName(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:

    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num) {return C_scenario_elem();};

    /*! @brief   フォーマットエラーメッセージの生成 
     *  @param   size_check      コマンドトークン列サイズチェック 
     *  @param   for_token_check FORコマンドトークン存在チェック 
     *  @param   val_check       値のトークン存在チェック 
     */
    void createFormatErrMsg(bool size_check,
                            bool for_token_check,
                            bool val_check);

};

#endif
