/*! 
    @file  C_labeldef_cmd_interpreter.h 
    @brief ラベル定義コマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_LABELDEF_CMD_INTERPRETER_H
#define C_LABELDEF_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"
#include <map>

/*! 
  ラベル定義コマンドインタプリタクラス 
  @brief  ラベル定義コマンドの解釈機能を持つ 
 */
class C_labeldef_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    bool m_check;
    bool m_labeldef;
public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_labeldef_cmd_interpreter(boost::shared_ptr<I_param> param,
                               boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_labeldef_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);


    /*! @brief   トークン列からラベル定義をチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @param   label_map ラベルマップ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool checkLabelDef(std::vector<std::string>::iterator begin,
                       std::vector<std::string>::iterator end,
                       std::map<std::string,std::pair<size_t,size_t> > &label_def_map);

    /*! @brief   トークン列からラベル名を取得する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  ラベル名 
     */
    std::string getLabelName(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:

    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num) {return C_scenario_elem();};

    /*! @brief   フォーマットエラーメッセージの生成 
     *  @param   size_check               コマンドトークン列サイズチェック 
     *  @param   labeldef_token_check     ":"トークン存在チェック 
     */
    void createFormatErrMsg(bool size_check,
                            bool labeldef_token_check);

    /*! @brief   ラベル定義エラーメッセージの生成 
     */
    void createLabelDefErrMsg();
};

#endif

