/*! 
    @file  C_param_parser.h 
    @brief パラメータファイルをパースするクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_PARAM_PARSER_H
#define C_PARAM_PARSER_H

#include <vector>
#include <string>
#include <boost/shared_ptr.hpp>
#include "C_parser_base.h"
#include "I_param.h"


/*! 
  パラメータファイルパーサクラス 
  @brief  パラメータファイルからシナリオクラスオブジェクト生成する機能を持つ 
 */
class C_param_parser : public C_parser_base {
public:
    /* public メンバ関数 */

    /*! @brief パースしたい文字列のvectorとパラメータクラスを設定します。 
     *  @param filename  ファイル名 
     *  @param param   パラメータクラス 
     */
    virtual void setData(std::string &filename,
                         boost::shared_ptr<I_param> &param);

    /*! @brief   パスからファイル名を取り出します。 
     *  @return  true: パース成功, false: パース失敗 
     */
    virtual bool parse();

private:
    /* private メンバ関数 */
    /* ここには、C_paramクラスのオブジェクト生成に関係する関数を記述 */
    /* 構文解析用の関数を記述 */

    /*! @brief   パラメータ定義終端(:)を探索する。 
     *  @param   begin 探索対象トークンの先頭を指すイテレータ 
     *  @param   end   探索対象トークンの終端を指すイテレータ 
     *  @return  パラメータ定義終端を指すイテレータ 
     */
    std::vector<std::string>::iterator
    findParamDefEndToken(std::vector<std::string>::iterator begin,
                         std::vector<std::string>::iterator end);


    /*! @brief   トークン列のパラメータ名定義終端の存在をチェックする 
     *  @param   begin          対象トークンの先頭を指すイテレータ 
     *  @param   end            対象トークンの終端を指すイテレータ 
     *  @param   line_num       beginが存在する行番号 
     *  @param   param_def_end  パラメータ名定義終端を指すイテレータ (成功時) 
     *  @return  true: 成功, false: 失敗 
     */
    bool checkParamDefEnd(std::vector<std::string>::iterator begin,
                          std::vector<std::string>::iterator end,
                          size_t line_num,
                          std::vector<std::string>::iterator &param_def_end);

    /*! @brief   パラメータ名トークン列をパースする 
     *  @param   begin      対象トークンの先頭を指すイテレータ 
     *  @param   end        対象トークンの終端を指すイテレータ 
     *  @param   line_num   beginが存在する行番号 
     *  @param   param_name パラメータ名 
     *  @return  true: パース成功, false: パース失敗 
     */
    bool parseParamToken(std::vector<std::string>::iterator begin,
                         std::vector<std::string>::iterator end,
                         size_t line_num,
                         std::vector<std::string> &param_name);

    /*! @brief   値トークン列をパースする 
     *  @param   begin      対象トークンの先頭を指すイテレータ 
     *  @param   end        対象トークンの終端を指すイテレータ 
     *  @param   line_num   beginが存在する行番号 
     *  @param   val_str    値の文字列 
     *  @return  true: パース成功, false: パース失敗 
     */
    bool parseValToken(std::vector<std::string>::iterator begin,
                       std::vector<std::string>::iterator end,
                       size_t line_num,
                       std::string &val_str);
};

#endif
