/*! 
    @file  C_sc_cmd_arg.cpp 
    @brief シナリオ要素の引数 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_sc_cmd_arg.h"

/*! @brief コンストラクタ 
 *  @param val  値 
 */
C_sc_cmd_arg::C_sc_cmd_arg(int *val)
    : m_type(C_sc_cmd_arg::INT), m_ip(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(unsigned int *val)
    : m_type(C_sc_cmd_arg::UNSIGNED_INT), m_uip(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(double *val)
    : m_type(C_sc_cmd_arg::DOUBLE), m_dp(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(std::string *val)
    : m_type(C_sc_cmd_arg::STRING), m_sp(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(std::complex<double> *val)
    : m_type(C_sc_cmd_arg::COMPLEX), m_cp(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(std::vector<int> *val)
    : m_type(C_sc_cmd_arg::INT_VEC), m_ivp(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(std::vector<unsigned int> *val)
    : m_type(C_sc_cmd_arg::UNSIGNED_INT_VEC), m_uivp(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(std::vector<double> *val)
    : m_type(C_sc_cmd_arg::DOUBLE_VEC), m_dvp(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(std::vector<std::string> *val)
    : m_type(C_sc_cmd_arg::STRING_VEC), m_svp(val)
{}

C_sc_cmd_arg::C_sc_cmd_arg(std::vector<std::complex<double> > *val) 
    : m_type(C_sc_cmd_arg::COMPLEX_VEC), m_cvp(val)
{}


/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(int &val) {
    if (m_type == C_sc_cmd_arg::INT) {
        val = *m_ip;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(unsigned int &val) {
    if (m_type == C_sc_cmd_arg::UNSIGNED_INT) {
        val = *m_uip;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(double &val) {
    if (m_type == C_sc_cmd_arg::DOUBLE) {
        val = *m_dp;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(std::string &val) {
    if (m_type == C_sc_cmd_arg::STRING) {
        val = *m_sp;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(std::complex<double> &val) {
    if (m_type == C_sc_cmd_arg::COMPLEX) {
        val = *m_cp;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(std::vector<int> &val) {
    if (m_type == C_sc_cmd_arg::INT_VEC) {
        val = *m_ivp;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(std::vector<unsigned int> &val) {
    if (m_type == C_sc_cmd_arg::UNSIGNED_INT_VEC) {
        val = *m_uivp;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(std::vector<double> &val) {
    if (m_type == C_sc_cmd_arg::DOUBLE_VEC) {
        val = *m_dvp;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(std::vector<std::string> &val) {
    if (m_type == C_sc_cmd_arg::STRING_VEC) {
        val = *m_svp;
        return true;
    }
    return false;
}

/*! @brief  値取得関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::getVal(std::vector<std::complex<double> > &val) {
    if (m_type == C_sc_cmd_arg::COMPLEX_VEC) {
        val = *m_cvp;
        return true;
    }
    return false;
}

/*! @brief  値設定関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::setVal(int val) {
    if (m_type == C_sc_cmd_arg::INT) {
        *m_ip = val;
        return true;
    }
    return false;
}

/*! @brief  値設定関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::setVal(unsigned int val) {
    if (m_type == C_sc_cmd_arg::UNSIGNED_INT) {
        *m_uip = val;
        return true;
    }
    return false;
}

/*! @brief  ポインタ設定関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::setPtr(int *val) {
    if (m_type == C_sc_cmd_arg::INT) {
        m_ip = val;
        return true;
    }
    return false;
}

/*! @brief  ポインタ設定関数 
 *  @param  val     値 
 *  @return true 成功、false 失敗 
 */
bool C_sc_cmd_arg::setPtr(unsigned int *val) {
    if (m_type == C_sc_cmd_arg::UNSIGNED_INT) {
        m_uip = val;
        return true;
    }
    return false;
}

/*! @brief   値の代入を行う 
 *  @param   arg 代入値 
 */
void C_sc_cmd_arg::assignVal(const C_sc_cmd_arg &rhs) {
    if (m_type == rhs.m_type) {
        switch (m_type) {
        case C_sc_cmd_arg::INT : 
            *m_ip = *rhs.m_ip;
            break;
        case C_sc_cmd_arg::UNSIGNED_INT : 
            *m_uip = *rhs.m_uip;
            break;
        case C_sc_cmd_arg::DOUBLE :
            *m_dp = *rhs.m_dp;
            break;
        case C_sc_cmd_arg::STRING : 
            *m_sp = *rhs.m_sp;
            break;
        case C_sc_cmd_arg::COMPLEX : 
            *m_cp = *rhs.m_cp;
            break;
        case C_sc_cmd_arg::INT_VEC : 
            *m_ivp = *rhs.m_ivp;
            break;
        case C_sc_cmd_arg::UNSIGNED_INT_VEC : 
            *m_uivp = *rhs.m_uivp;
            break;
        case C_sc_cmd_arg::DOUBLE_VEC : 
            *m_dvp = *rhs.m_dvp;
            break;
        case C_sc_cmd_arg::STRING_VEC :
            *m_svp = *rhs.m_svp;
            break;
        case C_sc_cmd_arg::COMPLEX_VEC :
            *m_cvp = *rhs.m_cvp;
            break;
        }
    }
    else {
        /* タイプエラーの例外をthrow */
        throw C_sc_cmd_arg_type_error();
    }
}


/*! @brief   比較演算を行う 
 *  @param   rhs  右辺 
 */
void C_sc_cmd_arg::addAndAssignVal(const C_sc_cmd_arg &rhs) {
    if (m_type == rhs.m_type) {
        switch (m_type) {
        case C_sc_cmd_arg::INT : 
            *m_ip += *rhs.m_ip;
            break;
        case C_sc_cmd_arg::UNSIGNED_INT : 
            *m_uip += *rhs.m_uip;
            break;
        case C_sc_cmd_arg::DOUBLE :
            *m_dp += *rhs.m_dp;
            break;
        case C_sc_cmd_arg::STRING : 
            *m_sp += *rhs.m_sp;
            break;
        case C_sc_cmd_arg::COMPLEX : 
            *m_cp += *rhs.m_cp;
            break;
        case C_sc_cmd_arg::INT_VEC : 
            addForEach(m_ivp, rhs.m_ivp);
            break;
        case C_sc_cmd_arg::UNSIGNED_INT_VEC : 
            addForEach(m_ivp, rhs.m_ivp);
            break;
        case C_sc_cmd_arg::DOUBLE_VEC : 
            addForEach(m_dvp, rhs.m_dvp);
            break;
        case C_sc_cmd_arg::STRING_VEC :
            addForEach(m_svp, rhs.m_svp);
            break;
        case C_sc_cmd_arg::COMPLEX_VEC :
            addForEach(m_cvp, rhs.m_cvp);
            break;
        }
    }
    else {
        /* タイプエラーの例外をthrow */
        throw C_sc_cmd_arg_type_error();
    }
}


/*! @brief vector型同士で相互加算させる関数 
 *  @param vec1   vector<T>型のオブジェクト(加算される) 
 *  @param vec2   vector<T>型のオブジェクト 
 */
void C_sc_cmd_arg::addForEach(std::vector<int> *vec1, std::vector<int> *vec2) {
    std::vector<int>::iterator ite1 = vec1->begin();
    std::vector<int>::iterator ite2 = vec2->begin();
    std::vector<int>::iterator end1 = vec1->end();
    std::vector<int>::iterator end2 = vec2->end();
    for (; (ite1 != end1 && ite2 != end2); ++ite1, ++ite2) {
        *ite1 += *ite2;
    }
}

void C_sc_cmd_arg::addForEach(std::vector<unsigned int> *vec1, std::vector<unsigned int> *vec2) {
    std::vector<unsigned int>::iterator ite1 = vec1->begin();
    std::vector<unsigned int>::iterator ite2 = vec2->begin();
    std::vector<unsigned int>::iterator end1 = vec1->end();
    std::vector<unsigned int>::iterator end2 = vec2->end();
    for (; (ite1 != end1 && ite2 != end2); ++ite1, ++ite2) {
        *ite1 += *ite2;
    }
}

void C_sc_cmd_arg::addForEach(std::vector<double> *vec1, std::vector<double> *vec2) {
    std::vector<double>::iterator ite1 = vec1->begin();
    std::vector<double>::iterator ite2 = vec2->begin();
    std::vector<double>::iterator end1 = vec1->end();
    std::vector<double>::iterator end2 = vec2->end();
    for (; (ite1 != end1 && ite2 != end2); ++ite1, ++ite2) {
        *ite1 += *ite2;
    }
}

void C_sc_cmd_arg::addForEach(std::vector<std::string> *vec1,
                              std::vector<std::string> *vec2) {
    std::vector<std::string>::iterator ite1 = vec1->begin();
    std::vector<std::string>::iterator ite2 = vec2->begin();
    std::vector<std::string>::iterator end1 = vec1->end();
    std::vector<std::string>::iterator end2 = vec2->end();
    for (; (ite1 != end1 && ite2 != end2); ++ite1, ++ite2) {
        *ite1 += *ite2;
    }
}

void C_sc_cmd_arg::addForEach(std::vector<std::complex<double> > *vec1,
                              std::vector<std::complex<double> > *vec2) {
    std::vector<std::complex<double> >::iterator ite1 = vec1->begin();
    std::vector<std::complex<double> >::iterator ite2 = vec2->begin();
    std::vector<std::complex<double> >::iterator end1 = vec1->end();
    std::vector<std::complex<double> >::iterator end2 = vec2->end();
    for (; (ite1 != end1 && ite2 != end2); ++ite1, ++ite2) {
        *ite1 += *ite2;
    }
}



/*! @brief   比較演算を行う 
 *  @param   rhs  右辺 
 *  @return  true 比較成功、false 比較失敗 
 */
bool C_sc_cmd_arg::operator==(const C_sc_cmd_arg &rhs) {
    if (m_type == C_sc_cmd_arg::INT && rhs.m_type == C_sc_cmd_arg::INT) {
        return (*m_ip == *rhs.m_ip);
    }
    else if (m_type == C_sc_cmd_arg::UNSIGNED_INT && rhs.m_type == C_sc_cmd_arg::UNSIGNED_INT) {
        return (*m_uip == *rhs.m_uip);
    }
    else if (m_type == C_sc_cmd_arg::DOUBLE && rhs.m_type == C_sc_cmd_arg::DOUBLE) {
        return (*m_dp == *rhs.m_dp);
    }
    else if (m_type == C_sc_cmd_arg::STRING && rhs.m_type == C_sc_cmd_arg::STRING) {
        return (*m_sp == *rhs.m_sp);
    }
    else {
        throw C_sc_cmd_arg_type_error();
    }
    return false;
}

/*! @brief   比較演算を行う 
 *  @param   rhs  右辺 
 *  @return  true 比較成功、false 比較失敗 
 */
bool C_sc_cmd_arg::operator!=(const C_sc_cmd_arg &rhs) {
    return !(*this == rhs);
}

/*! @brief   比較演算を行う 
 *  @param   rhs  右辺 
 *  @return  true 比較成功、false 比較失敗 
 */
bool C_sc_cmd_arg::operator<(const C_sc_cmd_arg &rhs) {
    if (m_type == C_sc_cmd_arg::INT && rhs.m_type == C_sc_cmd_arg::INT) {
        return (*m_ip < *rhs.m_ip);
    }
    else if (m_type == C_sc_cmd_arg::UNSIGNED_INT && rhs.m_type == C_sc_cmd_arg::UNSIGNED_INT) {
        return (*m_uip < *rhs.m_uip);
    }
    else if (m_type == C_sc_cmd_arg::DOUBLE && rhs.m_type == C_sc_cmd_arg::DOUBLE) {
        return (*m_dp < *rhs.m_dp);
    }
    else {
        throw C_sc_cmd_arg_type_error();
    }
    return false;
}

/*! @brief   比較演算を行う 
 *  @param   rhs  右辺 
 *  @return  true 比較成功、false 比較失敗 
 */
bool C_sc_cmd_arg::operator<=(const C_sc_cmd_arg &rhs) {
    if (m_type == C_sc_cmd_arg::INT && rhs.m_type == C_sc_cmd_arg::INT) {
        return (*m_ip <= *rhs.m_ip);
    }
    else if (m_type == C_sc_cmd_arg::UNSIGNED_INT && rhs.m_type == C_sc_cmd_arg::UNSIGNED_INT) {
        return (*m_uip <= *rhs.m_uip);
    }
    else if (m_type == C_sc_cmd_arg::DOUBLE && rhs.m_type == C_sc_cmd_arg::DOUBLE) {
        return (*m_dp <= *rhs.m_dp);
    }
    else {
        throw C_sc_cmd_arg_type_error();
    }
    return false;
}

/*! @brief   比較演算を行う 
 *  @param   rhs  右辺 
 *  @return  true 比較成功、false 比較失敗 
 */
bool C_sc_cmd_arg::operator>(const C_sc_cmd_arg &rhs) {
    return !(*this <= rhs);
}

/*! @brief   比較演算を行う 
 *  @param   rhs  右辺 
 *  @return  true 比較成功、false 比較失敗 
 */
bool C_sc_cmd_arg::operator>=(const C_sc_cmd_arg &rhs) {
    return !(*this < rhs);
}


/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isInt() {
    return (m_type == C_sc_cmd_arg::INT);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isUnsignedInt() {
    return (m_type == C_sc_cmd_arg::UNSIGNED_INT);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isDouble() {
    return (m_type == C_sc_cmd_arg::DOUBLE);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isString() {
    return (m_type == C_sc_cmd_arg::STRING);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isComplex() {
    return (m_type == C_sc_cmd_arg::COMPLEX);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isIntVec() {
    return (m_type == C_sc_cmd_arg::INT_VEC);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isUnsignedIntVec() {
    return (m_type == C_sc_cmd_arg::UNSIGNED_INT_VEC);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isDoubleVec() {
    return (m_type == C_sc_cmd_arg::DOUBLE_VEC);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isStringVec() {
    return (m_type == C_sc_cmd_arg::STRING_VEC);
}

/*! @brief   型の判定を行う。 
 *  @return  true 該当の型、false 該当の型ではない 
 */
bool C_sc_cmd_arg::isComplexVec() {
    return (m_type == C_sc_cmd_arg::COMPLEX_VEC);
}


/*! @breif 内部のメンバ変数を出力(debug) 
 */
std::string C_sc_cmd_arg::dump() {
    std::ostringstream  oss;

    switch (m_type) {
    case C_sc_cmd_arg::INT : 
        if (m_ip == NULL) oss << "(NULL)";
        else oss << *m_ip;
        break;
    case C_sc_cmd_arg::UNSIGNED_INT : 
        if (m_uip == NULL) oss << "(NULL)";
        else oss << *m_uip;
        break;
    case C_sc_cmd_arg::DOUBLE :
        if (m_dp == NULL) oss << "(NULL)";
        else oss << *m_dp;
        break;
    case C_sc_cmd_arg::STRING : 
        if (m_sp == NULL) oss << "(NULL)";
        else oss << *m_sp;
        break;
    case C_sc_cmd_arg::COMPLEX : 
        if (m_cp == NULL) oss << "(NULL)";
        else oss << *m_cp;
        break;
    case C_sc_cmd_arg::INT_VEC :
        if (m_ivp == NULL) oss << "(NULL)";
        else {
            for( size_t i = 0; i < m_ivp->size(); i++ ){
                if(i+1 < m_ivp->size()) oss << "[" << i << "]" << m_ivp->at(i) << ", " ;
                else                    oss << "[" << i << "]" << m_ivp->at(i);
            }
        }
        break;
    case C_sc_cmd_arg::UNSIGNED_INT_VEC :
        if (m_uivp == NULL) oss << "(NULL)";
        else {
            for( size_t i = 0; i < m_uivp->size(); i++ ){
                if(i+1 < m_uivp->size()) oss << "[" << i << "]" << m_uivp->at(i) << ", " ;
                else                    oss << "[" << i << "]" << m_uivp->at(i);
            }
        }
        break;
    case C_sc_cmd_arg::DOUBLE_VEC : 
        if (m_dvp == NULL) oss << "(NULL)";
        else {
            for( size_t i = 0; i < m_dvp->size(); i++ ){
                if(i+1 < m_dvp->size()) oss << "[" << i << "]" << m_dvp->at(i) <<  ", " ;
                else                    oss << "[" << i << "]" << m_dvp->at(i);
            }
        }
        break;
    case C_sc_cmd_arg::STRING_VEC :
        if (m_svp == NULL) oss << "(NULL)";
        else {
            for( size_t i = 0; i < m_svp->size(); i++ ){
                if(i+1 < m_svp->size()) oss << "[" << i << "]" << m_svp->at(i) <<  ", " ;
                else                    oss << "[" << i << "]" << m_svp->at(i);
            }
        }
        break;
    case C_sc_cmd_arg::COMPLEX_VEC :
        if (m_cvp == NULL) oss << "(NULL)";
        else {
            for( size_t i = 0; i < m_cvp->size(); i++ ){
                if(i+1 < m_cvp->size()) oss << "[" << i << "]" << m_cvp->at(i) <<  ", " ;
                else                    oss << "[" << i << "]" << m_cvp->at(i);
            }
        }
        break;
    }

    return oss.str();
}
