/*! 
    @file  C_sc_ptr_table.cpp 
    @brief シナリオポインタ管理クラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#include "C_sc_ptr_table.h"
#include <sstream>

/*! @brief   デストラクタ 
 */
C_sc_ptr_table::~C_sc_ptr_table() {
    clear();
}

/*! @brief   ポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clear() {
    clearIntTable();
    clearUnsignedIntTable();
    clearDoubleTable();
    clearStringTable();
    clearComplexTable();
    clearIntVecTable();
    clearUnsignedIntVecTable();
    clearDoubleVecTable();
    clearStringVecTable();
    clearComplexVecTable();
}

/*! @brief   intポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearIntTable() {
    std::vector<int *>::iterator ite = m_int_ptr_table.begin();
    std::vector<int *>::iterator end = m_int_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_int_ptr_table.clear();
}

/*! @brief   unsigned intポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearUnsignedIntTable() {
    std::vector<unsigned int *>::iterator ite = m_uint_ptr_table.begin();
    std::vector<unsigned int *>::iterator end = m_uint_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_uint_ptr_table.clear();
}

/*! @brief   doubleポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearDoubleTable() {
    std::vector<double *>::iterator ite = m_double_ptr_table.begin();
    std::vector<double *>::iterator end = m_double_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_double_ptr_table.clear();
}

/*! @brief   stringポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearStringTable() {
    std::vector<std::string *>::iterator ite = m_string_ptr_table.begin();
    std::vector<std::string *>::iterator end = m_string_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_string_ptr_table.clear();
}

/*! @brief   complex<double>ポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearComplexTable() {
    std::vector<std::complex<double> *>::iterator ite = m_complex_ptr_table.begin();
    std::vector<std::complex<double> *>::iterator end = m_complex_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_complex_ptr_table.clear();
}

/*! @brief   vector<int>ポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearIntVecTable() {
    std::vector<std::vector<int> *>::iterator ite = m_int_vec_ptr_table.begin();
    std::vector<std::vector<int> *>::iterator end = m_int_vec_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_int_vec_ptr_table.clear();
}

/*! @brief   vector<unsigned int>ポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearUnsignedIntVecTable() {
    std::vector<std::vector<unsigned int> *>::iterator ite = m_uint_vec_ptr_table.begin();
    std::vector<std::vector<unsigned int> *>::iterator end = m_uint_vec_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_uint_vec_ptr_table.clear();
}

/*! @brief   vector<double>ポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearDoubleVecTable() {
    std::vector<std::vector<double> *>::iterator ite = m_double_vec_ptr_table.begin();
    std::vector<std::vector<double> *>::iterator end = m_double_vec_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_double_vec_ptr_table.clear();
}

/*! @brief   vector<string>ポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearStringVecTable() {
    std::vector<std::vector<std::string> *>::iterator ite = m_string_vec_ptr_table.begin();
    std::vector<std::vector<std::string> *>::iterator end = m_string_vec_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_string_vec_ptr_table.clear();
}

/*! @brief   vector<complex<double> > ポインタテーブルをリセットする 
 */
void C_sc_ptr_table::clearComplexVecTable() {
    std::vector<std::vector<std::complex<double> > *>::iterator ite 
        = m_complex_vec_ptr_table.begin();
    std::vector<std::vector<std::complex<double> > *>::iterator end
        = m_complex_vec_ptr_table.end();
    for (; ite != end; ++ite) {
        delete *ite;
    }
    m_complex_vec_ptr_table.clear();
}


/*! @brief   ポインタをテーブルに追加する。 
 *  @param   ptr 追加対象のポインタ 
 */
void C_sc_ptr_table::addPtr(int *ptr) {
    m_int_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(unsigned int *ptr) {
    m_uint_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(double *ptr) {
    m_double_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(std::string *ptr) {
    m_string_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(std::complex<double> *ptr) {
    m_complex_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(std::vector<int> *ptr) {
    m_int_vec_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(std::vector<unsigned int> *ptr) {
    m_uint_vec_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(std::vector<double> *ptr) {
    m_double_vec_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(std::vector<std::string> *ptr) {
    m_string_vec_ptr_table.push_back(ptr);
}

void C_sc_ptr_table::addPtr(std::vector<std::complex<double> > *ptr) {
    m_complex_vec_ptr_table.push_back(ptr);
}

std::string C_sc_ptr_table::dump() {
    std::ostringstream oss("");

    oss << "int                      ptr table size : " << m_int_ptr_table.size() << "\n"
        << "unsigned int             ptr table size : " << m_uint_ptr_table.size() << "\n"
        << "double                   ptr table size : " << m_double_ptr_table.size() << "\n"
        << "string                   ptr table size : " << m_string_ptr_table.size() << "\n"
        << "complex<double>          ptr table size : " << m_complex_ptr_table.size() << "\n"
        << "vector<int>              ptr table size : " << m_int_vec_ptr_table.size() << "\n"
        << "vector<unsigned int>     ptr table size : " << m_uint_vec_ptr_table.size() << "\n"
        << "vector<double>           ptr table size : " << m_double_vec_ptr_table.size() << "\n"
        << "vector<string>           ptr table size : " << m_string_vec_ptr_table.size() << "\n"
        << "vector<complex<double> > ptr table size : " << m_complex_vec_ptr_table.size();

    return oss.str();
}
