/*! 
    @file  C_sc_ptr_table.h 
    @brief シナリオポインタ管理クラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SC_PTR_TABLE_H
#define C_SC_PTR_TABLE_H

#include <vector>
#include <complex>
#include <string>

/*! 
  シナリオパース時に生成したポインタを管理するクラス 
  @brief  シナリオパース時に生成したポインタを管理する機能を持つ 
 */
class C_sc_ptr_table {
private:
    /* int型ポインタテーブル */
    std::vector<int *> m_int_ptr_table;
    
    /* unsigned int型ポインタテーブル */
    std::vector<unsigned int *> m_uint_ptr_table;

    /* double型ポインタテーブル */
    std::vector<double *> m_double_ptr_table;

    /* string型ポインタテーブル */
    std::vector<std::string *> m_string_ptr_table;

    /* complex型ポインタテーブル */
    std::vector<std::complex<double> *> m_complex_ptr_table;

    /* vector<int>型ポインタテーブル */
    std::vector<std::vector<int> *> m_int_vec_ptr_table;
    
    /* vector<unsigned int>型ポインタテーブル */
    std::vector<std::vector<unsigned int> *> m_uint_vec_ptr_table;

    /* vector<double>型ポインタテーブル */
    std::vector<std::vector<double> *> m_double_vec_ptr_table;

    /* vector<string>型ポインタテーブル */
    std::vector<std::vector<std::string> *> m_string_vec_ptr_table;

    /* vector<complex<double> >型ポインタテーブル */
    std::vector<std::vector<std::complex<double> > *> m_complex_vec_ptr_table;
public:
    /*! @brief   デストラクタ 
     */
    ~C_sc_ptr_table();

    /*! @brief   ポインタテーブルをリセットする 
     */
    void clear();

    /*! @brief   ポインタをテーブルに追加する。 
     *  @param   ptr 追加対象のポインタ 
     */
    void addPtr(int *ptr);
    void addPtr(unsigned int *ptr);
    void addPtr(double *ptr);
    void addPtr(std::string *ptr);
    void addPtr(std::complex<double> *ptr);
    void addPtr(std::vector<int> *ptr);
    void addPtr(std::vector<unsigned int> *ptr);
    void addPtr(std::vector<double> *ptr);
    void addPtr(std::vector<std::string> *ptr);
    void addPtr(std::vector<std::complex<double> > *ptr);

    /*! @brief   内部の情報をdumpする 
     */
    std::string dump();
private:

    /*! @brief   intポインタテーブルをリセットする 
     */
    void clearIntTable();
    
    /*! @brief   unsigned intポインタテーブルをリセットする 
     */
    void clearUnsignedIntTable();

    /*! @brief   doubleポインタテーブルをリセットする 
     */
    void clearDoubleTable();

    /*! @brief   stringポインタテーブルをリセットする 
     */
    void clearStringTable();

    /*! @brief   complex<double>ポインタテーブルをリセットする 
     */
    void clearComplexTable();

    /*! @brief   vector<int>ポインタテーブルをリセットする 
     */
    void clearIntVecTable();
    
    /*! @brief   vector<unsigned int>ポインタテーブルをリセットする 
     */
    void clearUnsignedIntVecTable();

    /*! @brief   vector<double>ポインタテーブルをリセットする 
     */
    void clearDoubleVecTable();

    /*! @brief   vector<string>ポインタテーブルをリセットする 
     */
    void clearStringVecTable();

    /*! @brief   vector<complex<double> > ポインタテーブルをリセットする 
     */
    void clearComplexVecTable();
};

#endif
