/*! 
    @file  C_scdef_cmd_interpreter.h 
    @brief SCENARIOコマンドインタプリタクラス 
 
    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SCDEF_CMD_INTERPRETER_H
#define C_SCDEF_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"

/*! 
  SCENARIOコマンドインタプリタクラス 
  @brief  SCENARIOコマンドの解釈機能を持つ 
 */
class C_scdef_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    bool m_check;
    bool m_scdef;
public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_scdef_cmd_interpreter(boost::shared_ptr<I_param> param,
                            boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_scdef_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   トークン列からシナリオ名をチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @param   sc_map シナリオ名マップ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool checkScDef(std::vector<std::string>::iterator begin,
                    std::vector<std::string>::iterator end,
                    std::map<std::string,size_t> &sc_map);

    /*! @brief   トークン列からシナリオ名を取得する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ名 
     */
    std::string getScName(std::vector<std::string>::iterator begin,
                          std::vector<std::string>::iterator end);

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:
    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num) {return C_scenario_elem();};

    /*! @brief   フォーマットエラーメッセージの生成 
     *  @param   size_check               コマンドトークン列サイズチェック 
     *  @param   scdef_token_check        SCENARIOコマンドトークン存在チェック 
     */
    void createFormatErrMsg(bool size_check,
                            bool scdef_token_check);

    /*! @brief   シナリオ定義エラーメッセージの生成 
     */
    void createScDefErrMsg();

};

#endif

