/*! 
    @file  C_scend_cmd_interpreter.h 
    @brief SCENDコマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SCEND_CMD_INTERPRETER_H
#define C_SCEND_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"

/*! 
  SCENDコマンドインタプリタクラス 
  @brief  SCENDコマンドの解釈機能を持つ 
 */
class C_scend_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    bool m_check;
public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_scend_cmd_interpreter(boost::shared_ptr<I_param> param,
                            boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_scend_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);


    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num);
    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:

    /*! @brief   フォーマットエラーメッセージの生成 
     *  @param   size_check            コマンドトークン列サイズチェック 
     *  @param   end_token_check       ENDコマンドトークン存在チェック 
     */
    void createFormatErrMsg(bool size_check,
                            bool end_token_check);
};

#endif

