/*! 
    @file  C_subdef_cmd_interpreter.h 
    @brief SUBコマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_SUBDEF_CMD_INTERPRETER_H
#define C_SUBDEF_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"

/*! 
  SUBコマンドインタプリタクラス 
  @brief SUBコマンドの解釈機能を持つ 
 */
class C_subdef_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    bool m_check;
    bool m_subdef;
public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_subdef_cmd_interpreter(boost::shared_ptr<I_param> param,
                             boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_subdef_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   トークン列からサブルーチン定義をチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @param   sub_map サブルーチンマップ 
     *  @return  true エラーなし、false エラーあり 
     */
    bool checkSubDef(std::vector<std::string>::iterator begin,
                     std::vector<std::string>::iterator end,
                     std::map<std::string,std::pair<size_t,size_t> > &sub_map);


    /*! @brief   トークン列からサブルーチン名を取得する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  サブルーチン名 
     */
    std::string getSubName(std::vector<std::string>::iterator begin,
                           std::vector<std::string>::iterator end);

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:
    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num) {return C_scenario_elem();};

    /*! @brief   フォーマットエラーメッセージの生成 
     *  @param   size_check               コマンドトークン列サイズチェック 
     *  @param   subdef_token_check       SUBコマンドトークン存在チェック 
     */
    void createFormatErrMsg(bool size_check,
                            bool subdef_token_check);

    /*! @brief   サブルーチン定義エラーメッセージの生成 
     */
    void createSubDefErrMsg();
};

#endif

