/*! 
    @file  C_tdset_cmd_interpreter.h 
    @brief 実行時刻直接設定コマンドインタプリタクラス 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef C_TDSET_CMD_INTERPRETER_H
#define C_TDSET_CMD_INTERPRETER_H

#include "C_sc_cmd_interpreter.h"

/*! 
  実行時刻直接設定コマンドインタプリタクラス 
  @brief  実行時刻直接設定コマンドの解釈機能を持つ 
 */
class C_tdset_cmd_interpreter : public C_sc_cmd_interpreter {
private:
    /* メンバ変数 */
    enum E_tdset_type {
        NON = 0,
        TIME,
        FRAME
    };

    int m_type;
    bool m_check;
public:
    /* public メンバ関数 */

    /*! @brief   コンストラクタ 
     *  @param   param パラメータ 
     *  @param   sc_ptr_talbe  ポインタテーブル 
     */
    C_tdset_cmd_interpreter(boost::shared_ptr<I_param> param,
                            boost::shared_ptr<C_sc_ptr_table> sc_ptr_table);
    
    /*! @brief   デストラクタ 
     */
    virtual ~C_tdset_cmd_interpreter() {};

    /*! @brief   インタプリタの状態をリセットする。 
     */
    virtual void reset();

    /*! @brief   トークン列からコマンドフォーマットをチェックする 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  true エラーなし、false エラーあり 
     */
    virtual bool checkFormat(std::vector<std::string>::iterator begin,
                             std::vector<std::string>::iterator end);

    /*! @brief   シナリオ要素を生成する 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  シナリオ要素 
     */
    virtual C_scenario_elem createCmd(std::vector<std::string>::iterator begin,
                                      std::vector<std::string>::iterator end,
                                      int line_num);

    /*! @brief   @TIMEへの設定コマンドかどうかを返す。 
     *  @param   begin 対象トークンの先頭を指すイテレータ 
     *  @param   end   対象トークンの終端を指すイテレータ 
     *  @return  1 @TIMEへの設定コマンド、0 @FRAMEへの設定コマンド 
     */
    bool isTimeSetCmd(std::vector<std::string>::iterator begin,
                      std::vector<std::string>::iterator end);

    /*! @brief   内部情報のダンプ 
     *  @return  内部情報 
     */
    virtual std::string dump();

private:

    /*! @brief   フォーマットエラーメッセージの生成 
     *  @param   size_check        トークン列のサイズチェック結果 
     *  @param   time_token_check  TIMEトークンの存在チェック結果 
     *  @param   frame_token_check FRAMEトークンの存在チェック結果 
     *  @param   val_check         値のチェック結果 
     */
    void createFormatErrMsg(bool size_check,
                            bool time_token_check,
                            bool frame_token_check,
                            bool val_check);


};

#endif

