/*! 
    @file  C_tokenizer.cpp 
    @brief トークン作成機クラス  

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/
#include "C_tokenizer.h"
#include <boost/spirit/include/classic_core.hpp>
#include <boost/spirit/include/classic_utility.hpp>

/*! @brief       分割したい文字列を設定する 
 *  @param lines トークン分割したい文字列 
 */
void C_tokenizer::setLine(std::string &lines){
    m_str = lines;
}

/*! @brief       分割後のトークンを返す 
 *  @return      tokenのvectorを返す 
 */
std::vector<std::string> C_tokenizer::getTokens(){
    /* m_strを分割 */
    m_tokenizer.assign(m_str);

    /* 分割後のトークン出力buffer */
    std::vector<std::string> buf(m_tokenizer.begin(), m_tokenizer.end());

    return buf;
}



/***************************************************************** 
 *  C_tokenizer::separatorクラス 
 *****************************************************************/
/*! @brief       指数の判別 
 *  @param next  文字列探索箇所を指すイテレータ 
 *  @param end   文字列終端を指すイテレータ 
 *  @return      true: 指数、false: 指数以外 
 */
template <typename Iterator, typename Token>
bool C_tokenizer::separator::checkExponential(Iterator& next, Iterator& end, Token& tok){
    if( tok.empty() ){
        /* トークン内が空 */
        return false;
    }
    else if( !('0' <= *(next+1) && *(next+1) <= '9') ){
        /* 次の文字が数値以外 */
        return false;
    }
    else if( tok.find_last_of("eE") != tok.size()-1){
        /* 指数表現無し */
        return false;
    }

    std::string     buf = tok;
    Iterator        it  = next;

    /* +を追加してインクリメント */
    buf += *it;
    ++it;
    while( it != end ){
        /* , ; 比較記号、空白文字が来るまで取得 */
        if( *it == ',' || *it == ';' || *it == '+' || *it == '=' || *it == '!'  ||
            *it == '<' || *it == '>' || *it == ' ' || *it == '\t'|| *it == '\r' || *it == '\n' )
            break;

        buf += *it;
        ++it;
    }

    /* 指数チェック */
    boost::spirit::classic::parse_info<> info = boost::spirit::classic::parse( buf.c_str(), boost::spirit::classic::real_p );

    return info.full;
}
