/*! 
    @file  OneBlockBase.cpp
    @brief DSP関係の共通クラス定義(アルゴリズム呼び出し) 

    @date     $Date:: 2016-06-10 12:47:02 #$
    @revision $Revision: 242 $

    Copyright (C) 2015, 2016 Fujitsu Limited. All rights reserved.
*/

#include "C_OneBlockBase.h"

//########################################################### 
// class OneBlockBase 
//########################################################### 

// MCC_FUJITSU Execute I/F 
// TS_SYNC_FUJITSU Execute I/F 
void OneBlockBase::Execute(const double             dsptm,
                           const bool                  FrameTopIn,
                           const std::complex<double>& InputData_X,
                           const std::complex<double>& InputData_Y) {
  m_algo->Execute(dsptm, FrameTopIn, InputData_X, InputData_Y) ;
}

// FMR_FUJITSU  Execute I/F 
void OneBlockBase::Execute( const double                dsptm,
                            const std::complex<double>& InputData_H,
                            const std::complex<double>& InputData_V,
                                  std::complex<double>& OutputData_H,
                                  std::complex<double>& OutputData_V ) {
  m_algo->Execute(dsptm, InputData_H, InputData_V, OutputData_H, OutputData_V);
}

// CPR_FUJITSU  Execute I/F 
// NPCC_FUJITSU Execute I/F 
void OneBlockBase::Execute( const double                dsptm,
                            const bool                  FrameTopIn,
                            const std::complex<double>& InputData_H,
                            const std::complex<double>& InputData_V,
                                  bool&                 FrameTopOut,
                                  std::complex<double>& OutputData_H,
                                  std::complex<double>& OutputData_V ) {
  m_algo->Execute(dsptm, FrameTopIn, InputData_H, InputData_V, FrameTopOut, OutputData_H, OutputData_V);
}

// DEC_FUJITSU Execute I/F 
void OneBlockBase::Execute( const double                dsptm,
                            const bool                  FrameTopIn,
                            const std::complex<double>& InputData_H,
                            const std::complex<double>& InputData_V,
                                  bool&                 FrameTopOut,
                                  std::vector<unsigned int>& OutputBitData) {
  m_algo->Execute(dsptm, FrameTopIn, InputData_H, InputData_V, FrameTopOut, OutputBitData);
}

// PHA_FUJITSU Execute I/F 
void OneBlockBase::Execute(const double                dsptm,
                           const double&               AdjustPhase,
                           const std::complex<double>& InputData_H,
                           const std::complex<double>& InputData_V,
                                 bool&                 IfftFirstData,
                                 std::complex<double>& OutputData_H,
                                 std::complex<double>& OutputData_V ) {
  m_algo->Execute(dsptm, AdjustPhase, InputData_H, InputData_V,
                  IfftFirstData, OutputData_H, OutputData_V) ;
}

// SS_FUJITSU Execute I/F 
void OneBlockBase::Execute(const double                dsptm,
                           const int                   VcoInitialTraining,
                           const unsigned int          PhaP_OffsetOn,
                           const bool                  PhaFirstData,
#ifndef rel_NTT
                           const unsigned int          rxrefck,
#endif
                           const std::complex<double>& InputData_H,
                           const std::complex<double>& InputData_V,
                                 bool&                 AdjustPhaseOutTiming,
                                 double&               AdjustPhase,
                                 double&               ExtCtrl,
                                 bool&                 VcoFeedbackTiming,
                                 double&               VcoFeedback,
                                 double&               ClkVcoDo) {
  m_algo->Execute(dsptm,
                  VcoInitialTraining,
                  PhaP_OffsetOn,
                  PhaFirstData,
#ifndef rel_NTT
                  rxrefck,
#endif
                  InputData_H,
                  InputData_V,
                  AdjustPhaseOutTiming,
                  AdjustPhase,
                  ExtCtrl,
                  VcoFeedbackTiming,
                  VcoFeedback,
                  ClkVcoDo) ;
}

// SQM_FUJITSU Execute I/F 
void OneBlockBase::Execute( const double                dsptm,
                            const bool                  FrameTopIn,
                            const std::complex<double>& InputData_H,
                            const std::complex<double>& InputData_V,
                                  bool&                 Calc_Status) {
  m_algo->Execute(dsptm, FrameTopIn, InputData_H, InputData_V, Calc_Status);
}

#ifndef rel_NTT
// MCC_FUJITSU Execute I/F 
void OneBlockBase::Execute(const double             dsptm,
                           const bool                  FrameTopIn,
                           const std::complex<double>& InputData_X,
                           const std::complex<double>& InputData_Y,
                           int*  monitor) {
  m_algo->Execute(dsptm, FrameTopIn, InputData_X, InputData_Y, monitor) ;
}
#endif

// FFT Execute I/F
void OneBlockBase::Execute(const std::vector<std::complex<double> >& InputData,
                           std::vector<std::complex<double> >& OutputData,
                           const bool ifft) {
    m_algo->Execute(InputData, OutputData, ifft);
}

// DTAT_GEN Execute I/F
void OneBlockBase::Execute(std::vector<std::complex<double> >& OutputData) {
    m_algo->Execute(OutputData);
}


void OneBlockBase::Get( double& Pow_Ave_H,
                        double& Dif_Ave_H,
                        double& Pow_Ave_V,
                        double& Dif_Ave_V) {
  m_algo->Get(Pow_Ave_H, Dif_Ave_H, Pow_Ave_V, Dif_Ave_V);
}

void OneBlockBase::SetAeqTsTiming(const double ats_timing_h,
                                  const double ats_timing_v) {
  m_algo->SetAeqTsTiming(ats_timing_h, ats_timing_v) ;
}

void OneBlockBase::SetFrequencyOffset(const double fo_h_in,
                                      const double fo_v_in) {
  m_algo->SetFrequencyOffset(fo_h_in, fo_v_in) ;
}

void OneBlockBase::read_PD_MON_FOR_FEQ(double& val, bool& flg) {
  m_algo->read_PD_MON_FOR_FEQ(val, flg) ;
}


//########################################################### 
// class OneBlockBaseTx 
//########################################################### 

// MAP_FUJITSU Execute I/F 
void OneBlockBaseTx::Execute(const double                dsptm,
                             const bool                  ClockTop,
                             const struct OneBlockBaseTx::STxDspInput&   Input,
                                   std::complex<double>& OutputData_X,
                                   std::complex<double>& OutputData_Y) {
    m_algo->Execute(dsptm, ClockTop, Input, OutputData_X, OutputData_Y) ;
}
// TSS_FUJITSU Execute I/F 
// FC_FUJITSU Execute I/F 
// LIN_FUJITSU Execute I/F 
// EQL_FUJITSU Execute I/F 
void OneBlockBaseTx::Execute(const double                dsptm,
                             const bool                  ClockTop,
                             const std::complex<double>& InputData_X,
                             const std::complex<double>& InputData_Y,
                                   std::complex<double>& OutputData_X,
                                   std::complex<double>& OutputData_Y) {
    m_algo->Execute(dsptm, ClockTop, InputData_X, InputData_Y, OutputData_X, OutputData_Y) ;
}
// PreCoder_FUJITSU Execute I/F 
void OneBlockBaseTx::Execute(const double                dsptm,
                             const struct OneBlockBaseTx::STxDspInput&   Input,
                                   struct OneBlockBaseTx::STxDspInput&   Output,
                                   bool&                 ClockTop) {
    m_algo->Execute(dsptm, Input, Output, ClockTop) ;
}
