/*! 
    @file  checkLicense.h 
    @brief ライセンスチェック(秘匿化部分) 

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef CHECK_LICENSE_H
#define CHECK_LICENSE_H


/* [説明書] 
 *   このファイルは、ライセンスチェックを行いたいアーカイブ毎にincludeして 
 *   使用します。 
 *   ただし、Link時に重複しないように、ユニークなnamespace内でincludeして 
 *   ご利用ください。 
 *   
 *   [例] 
 *   ・アーカイブA 
 *     - ソースの先頭 
 *       namespace ARCHIVE_A { 
 *         #include "checkLicense.h" 
 *       } 
 *      
 *     - コンストラクタ等 
 *       A::A { 
 *         ARCHIVE_A::checkLicense() ; ライセンスチェック 
 *         ・・・以下略・・・ 
 */

#include <time.h>
#include <iostream>
#include <string>

#define LICENSE_LIMIT_YEAR   2029
#define LICENSE_LIMIT_MONTH  8
#define LICENSE_LIMIT_DAY    31

void checkLicense() {
    /*  現在の時刻を取得 */
    time_t     current_time = time(NULL) ;
  
    /*  年／月／日を tm 型に入れて、time_t 型に変換 */
    struct tm  end_time_tm = {0};
    end_time_tm.tm_year = LICENSE_LIMIT_YEAR  - 1900 ;
    end_time_tm.tm_mon  = LICENSE_LIMIT_MONTH - 1 ;
    end_time_tm.tm_mday = LICENSE_LIMIT_DAY + 1 ; /* 当日は起動OKとしたいので、+1 */
    end_time_tm.tm_hour = 0 ;
    end_time_tm.tm_min  = 0 ;
    end_time_tm.tm_sec  = 0 ;
  
    time_t     end_time_t = mktime(&end_time_tm) ;
  
    if (current_time > end_time_t) {
        /* 期限が過ぎている 
         *  恐らく、ErrorMngクラスがインスタンス中に発生するはずなので、 
         *   cerrで出力。 */
        std::cerr << "Error: The time limit has been passed." << std::endl ;
        exit(16) ;
    } else {
        /* まだ大丈夫 */
    }
    return ;
}
#endif
