/*! 
    @file  checkMachineLinux.h 
    @brief linux用実行マシンチェック (if need)  

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef CHECK_MACHINE_LINUX_H
#define CHECK_MACHINE_LINUX_H


/* [説明書] 
 *   bool checkMachine()は、実行PCのLoopbackとPoint-to-Point以外の全IFのうち 
 *   どれか一つでもMACアドレスリストのものと一致したら、実行可能マシンと判定。 
 *   (戻り値がfalseならば、判定エラー) 
 *   main.cppの先頭で使用してください。 
 *   
 *   [MACアドレス設定例] 
 *   ・c_mac_address_list へ、許可したいMACアドレスを追加。 
 *   ・#define CHECK_MAC_ADDRESS_100G_LNX がコメントアウトされていない事を確認。 
 *    
 *    ※ MACアドレスのチェックは行わない場合は、 
 *       #define CHECK_MAC_ADDRESS_100G_LNXをコメントアウトして、 
 *       c_mac_address_list[] = { "NotCheckMacAddress" } を定義する。 
 */

/* 
#define CHECK_MAC_ADDRESS_100G_LNX 
*/

/* チェックするIF数の最大値を下記defineで定義する 
 * (下記IF数よりも多い場合、それらはチェック対象外(無視される)) 
 */
#define MAX_IF_NUM 256

#include <iostream>
#include <string>
#include <string.h>

#include <sstream>
#include <iomanip>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>

const std::string c_mac_address_list[] = {
    #ifdef CHECK_MAC_ADDRESS_100G_LNX
    "00:05:E5:58:53:50",
    "00:03:E5:88:44:50",
    "00:05:E4:85:5E:D5"
    #else
    "NotCheckMacAddress"
    #endif
};

bool checkMachine() 
{
    bool LOGON_Allow = false;
    /* MACアドレスをCHECKしない場合 */
    if(c_mac_address_list[0] == "NotCheckMacAddress") {
        return true;
    }
    /*  MACアドレスの取得  */
    std::string get_mac;
    int fd;
    fd = socket(AF_INET, SOCK_DGRAM, 0);
    /* IF名の取得 */
    struct ifconf ifc;
    struct ifreq ifr_names[MAX_IF_NUM];
    ifc.ifc_len = sizeof(ifr_names);
    ifc.ifc_req = ifr_names;
    if (ioctl(fd, SIOCGIFCONF, &ifc) != 0) {
        return false;
    }
    for (int ii=0; !LOGON_Allow && ii<ifc.ifc_len/sizeof(ifr_names[0]); ii++) {
        struct ifreq ifr;
        /* デバイスのフラグの取得 */
        strncpy(ifr.ifr_name, ifr_names[ii].ifr_name, IFNAMSIZ-1);
        if (ioctl(fd, SIOCGIFFLAGS, &ifr) != 0) {
            continue;
        } else {
            if ((ifr.ifr_flags & (IFF_LOOPBACK | IFF_POINTOPOINT)) != 0) {
                /* LoopbackとPoint-to-Pointは無視 */
                continue;
            }
        }
        /* MACアドレスの取得 */
        ifr.ifr_addr.sa_family = AF_INET;
        strncpy(ifr.ifr_name, ifr_names[ii].ifr_name, IFNAMSIZ-1);
        if(!ioctl(fd, SIOCGIFHWADDR, &ifr)){
            std::ostringstream oss;
            oss.setf(std::ios::basefield);
            oss.setf(std::ios::uppercase);
            for (int i = 0; i < 6; i++) {
                oss << std::setw(2) << std::setfill('0') << std::hex
                    << static_cast<unsigned int>(static_cast<unsigned char>(ifr.ifr_hwaddr.sa_data[i]));
                if(i != 5) {
                    oss  << ':';
                }
            }
            std::string str(oss.str());
            get_mac = str;
        }
        /*
           std::cout << get_mac << std::endl;
        */
        for(size_t ii=0; ii < sizeof(c_mac_address_list)/sizeof(c_mac_address_list[0]); ii++) {
            if(get_mac == c_mac_address_list[ii]) {
                LOGON_Allow = true;
                break;
            }
        }
    }
    close(fd);
    return LOGON_Allow;
}

#endif  /* CHECK_MACHINE_LINUX_H */
