/*! 
    @file  checkMachineWin.h 
    @brief windows用実行マシンチェック (if need)   

    @date     $Date:: 2016-01-28 13:10:50 #$
    @revision $Revision: 2 $

    Copyright (C) 2015 Fujitsu Limited. All rights reserved.
*/

#ifndef CHECH_MACHINE_WIN_H
#define CHECH_MACHINE_WIN_H


/* [説明書] 
 *   bool checkMachine()は、実行PCのMACアドレスを取得して、実行可能マシンかを 
 *   判定する。(戻り値がfalseならば、判定エラー) 
 *   main.cppの先頭で使用してください。 
 *   
 *   [MACアドレス設定例] 
 *   ・c_mac_address_list へ、許可したいMACアドレスを追加。 
 *   ・#define CHECK_MAC_ADDRESS_100G がコメントアウトされていない事を確認。 
 *    
 *    ※ MACアドレスのチェックは行わない場合は、 
 *       #define CHECK_MAC_ADDRESS_100G をコメントアウトして、 
 *       c_mac_address_list[] = { "NotCheckMacAddress" } を定義する。 
 */

/* 
#define CHECK_MAC_ADDRESS_100G 
*/

#include <iostream>
#include <string>

#include <sstream>
#include <iomanip>
#include <winsock2.h>
#include <iphlpapi.h>
#pragma comment(lib,"iphlpapi.lib") 
#include <stdlib.h>

const std::string c_mac_address_list[] = {
    #ifdef CHECK_MAC_ADDRESS_100G
    "00-1F-83-B4-6A-D1",
    "00-19-99-7C-88-E5",
    "00-1F-73-B3-66-D3"
    #else
    "NotCheckMacAddress"
    #endif
};

bool checkMachine() 
{
    bool LOGON_Allow = false;
    /* MACアドレスをCHECKしない場合 */
    if(c_mac_address_list[0] == "NotCheckMacAddress") {
        return true;
    }
    /*  MACアドレスの取得  */
    std::string get_mac;
    ULONG len = 0;   
    DWORD ret = GetAdaptersAddresses(AF_UNSPEC, 0, 0, 0, &len);
    PIP_ADAPTER_ADDRESSES adpts_base = (PIP_ADAPTER_ADDRESSES)new BYTE[len];
    PIP_ADAPTER_ADDRESSES adpts = adpts_base;
    /* ここでadptsに情報が入る */
    for(ret = GetAdaptersAddresses(AF_UNSPEC, 0, 0, adpts, &len);adpts->PhysicalAddressLength != 0;adpts = adpts->Next) {
        /* PhysicalAddress, PhysicalAddressLengthメンバにより 
         * アダプタの物理アドレス（MACアドレス）を表示する    */
        if(adpts->PhysicalAddressLength != 0) {
            /* printf("  PhysicalAddress : "); */
            std::ostringstream oss;
            oss.setf(std::ios::hex, std::ios::basefield);
            oss.setf(std::ios::uppercase);
            for (DWORD i = 0; i < adpts->PhysicalAddressLength; i++) {
                oss << std::setw(2) << std::setfill('0') << std::hex << static_cast<DWORD>(adpts->PhysicalAddress[i]);
                if(i != adpts->PhysicalAddressLength - 1) {
                    oss  << '-';
                }
            }
            std::string str(oss.str());
            get_mac = str;
        }
        /* std::cout << get_mac << std::endl; */

        for(size_t ii=0; ii < sizeof(c_mac_address_list)/sizeof(c_mac_address_list[0]); ii++) {
            if(get_mac == c_mac_address_list[ii]) {
                LOGON_Allow = true;
            }
        }
    }
    delete[] adpts_base;

    return LOGON_Allow;
}

#endif  /* CHECH_MACHINE_WIN_H */
