// ---------------------------------------------------------------------------
//  100G  Function MODEL $Name:  $
// ---------------------------------------------------------------------------
/*!
 *  @file
 *  $Id: error_test_normal.cpp 2 2012-09-18 04:02:01Z kitajima $
 *  @brief エラー管理クラス(秘匿化部分)のテストコード
 */
// ---------------------------------------------------------------------------
// Copyright (c) FATEC Limited 2009
// ---------------------------------------------------------------------------
#include "ErrorMng_wrap.h"

/* [テスト内容] 
 *  以下の内容を確認します。
 *  - getVender() でVender名が取得できる事
 *  - getMakeDate() でアーカイブの作成日付が取得できる事
 *  - 各セヴィリティのメッセージが出力できる事
 *  - checkStatus()でセヴィリティがINFO、WARNINGの際にシステムを止めない事
 *  - checkStatus()でセヴィリティがERRORの際にシステムを止める事
 * 
 *  - ライセンスチェックの確認
 *    checkLicense.hのdefineを書き換えてテストする必要があるので、
 *    TP内にテストコードは特に存在しません。
 *    ＃以下のパタンをテストします。
 *         年  月  日             期待値
 *       (当日からの差)
 *      -  +0  +0  +0 (当日)   ライセンスOK
 *      -  +0  +0  +1          ライセンスOK
 *      -  +0  +1  +0          ライセンスOK
 *      -  +1  +0  +0          ライセンスOK
 *      -  +0  +0  +1          ライセンスOK
 *      -  +0  +1  -1          ライセンスOK
 *      -  +1  -1  -1          ライセンスOK
 *      -  +0  +0  -1          ライセンスNG
 *      -  +0  -1  +0          ライセンスNG
 *      -  -1  +0  +0          ライセンスNG
 * 
 * [テスト対象外（参考）]
 *  システム終了時に出力する最大エラーコードに対するテストは、
 *  error_test_"セヴィリティ名".cppで其々行いますので、ここではテスト
 *  しません。
 */
 
boost::shared_ptr<ErrorMng_wrap> g_err = createErrorMng() ;

int main() {

  std::cout << " vender    : " << g_err->getVender() << std::endl ;
  std::cout << " make time : " << g_err->getMakeTime() << std::endl ;
  
  
  g_err->appendMsg("I message", INFO_MSG) ;
  g_err->checkStatus() ;
  g_err->appendMsg("W message", WARNING_MSG) ;
  g_err->checkStatus() ;
  g_err->appendMsg("E message", ERROR_MSG) ;
  g_err->checkStatus() ;
  
  /*!
   * E messege を出力した後のcheckStatus()でシステムが止まるはずなので、
   * 下記coutの文字列が出力された場合、テストNGです。
   */
  std::cout << "*********************************" << std::endl ;
  std::cout << "* ErrorMng::checkStatus() is NG *" << std::endl ;
  std::cout << "*********************************" << std::endl ;
}

/*
  $Log: error_test_normal.cpp,v $
  Revision 1.1.1.1  2009/09/26 07:59:49  nakayama
  initial import

*/
