/*!
    @file  main.cpp
    @brief simulation main 

    @date     $Date:: 2016-12-08 20:43:35 #$
    @revision $Revision: 1070 $

    Copyright (C) 2015, 2016 Fujitsu Limited. All rights reserved.
*/

#include <time.h>
#include <string>
#include <complex>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <boost/shared_ptr.hpp>

#include "main.h"
#include "C_param.h"
#include "I_Util.h"
#include "I_param.h"
#include "C_scenario.h"
#include "C_param_parser.h"
#include "C_sc_parser.h"

#include "DATA_GEN_wrap.h"
#include "FFT_wrap.h"

#include <cmath>
#include "common.h"

boost::shared_ptr<ErrorMng_wrap> g_err = createErrorMng() ;

int main( int argc, char** argv) {


    std::string pfiles, str;
    
    /* 標準出力へメッセージを出力 */
    g_err->setOstreamOne(&std::cout) ;

    /* argument check: more than 1 */
    if (argc == 1) {
        std::stringstream msg;
        msg << "usage: > sim_100G param0.txt [param1.txt] [scenario.txt]" << std::endl;
        g_err->appendMsg(msg.str());
        g_err->checkStatus();
    }

    /* 解析エラーメッセージ保存用変数 */
    std::vector<std::string> errmsg;
    /* 共通クラスの生成 */
    boost::shared_ptr<I_Util> util = createI_Util();
    /* パラメータ名 */
    std::vector<std::string> paramName;
    /* シナリオ名 */
    std::string scName;

    /* コマンド引数の処理 */
    bool result = util->readArgs(argc, argv, paramName, scName, errmsg);

    /* ファイルの読み込み失敗 */
    if (!result) {
        for (std::vector<std::string>::iterator itrErr = errmsg.begin();
             itrErr != errmsg.end(); ++itrErr)
            g_err->appendMsg(*itrErr);
        g_err->checkStatus();
    }

    /* パラメータの構築 */
    /* パラメータクラスの生成 */
    boost::shared_ptr<I_param> param = createI_param();
    /* パラメータ解析クラスの生成 */
    boost::shared_ptr<C_param_parser> param_parser = createC_param_parser();

    /* ファイル名を出力用に文字列に変換 */
    for (std::vector<std::string>::iterator itrName = paramName.begin();
         itrName != paramName.end(); ++itrName)
        pfiles += *itrName + " ";
    pfiles += scName;

    /* パラメータファイル読み込み */
    result = util->readParamFile(param, param_parser, paramName, errmsg);

    /* パラメータの読み込み失敗 */
    if (!result) {
        for (std::vector<std::string>::iterator itrErr = errmsg.begin();
             itrErr != errmsg.end(); ++itrErr)
            g_err->appendMsg(*itrErr);
        g_err->checkStatus();
    }

    boost::shared_ptr<DATA_GEN_wrap> data_gen = boost::shared_ptr<DATA_GEN_wrap>(new DATA_GEN_wrap(param));
    g_err->checkStatus();
    boost::shared_ptr<FFT_wrap> fft = boost::shared_ptr<FFT_wrap>(new FFT_wrap(param));
    g_err->checkStatus();

    std::vector<std::complex<double> > InputData;
    std::vector<std::complex<double> > OutputDataFft;
    std::vector<std::complex<double> > OutputDataIfft;
    

    data_gen->Execute(InputData);
    fft->Execute(InputData, OutputDataFft, false);       // FFT処理
    //fft->Execute(OutputDataFft, OutputDataIfft, true);   // IFFT処理


    std::ofstream* os = new std::ofstream;
    os->open("log.txt");
    const double NowTime = 0.0;
    fft->ReportStatisticalData(*os, NowTime);
    
    std::cout << "Simulation End." << std::endl;
    return EXIT_SUCCESS;
}
